"""
`api_interface_test`
=======================================================================
Unit tests for code to upload XML files to the API
* Author(s): Bryan Siepert
"""
from embedops_cli.config import settings
from embedops_cli.eotools import post_test_report
import pytest


class FakeResponse(object):
    """Fake http response for test use"""

    status_code = 200

    def json(self):
        """Empty JSON object"""
        return "{}"


# test the cli/click to report_test fn call
class APIInterfaceTest:
    @pytest.fixture(autouse=True)
    def configure_env(self, monkeypatch, mocker):
        monkeypatch.setenv("CI", "true")
        monkeypatch.setenv("EMBEDOPS_API_REPO_KEY", "special_pwefix")
        self.logger_mock = mocker.patch("embedops_cli.eotools.ci_run._logger")

    def test_report_file_contents_sent(self, mocker):
        """test the cli/click to post_test_report fn call"""
        report_filename = "tests/file_fixtures/gtest_failure.xml"
        file_obj = open(report_filename, "rb")
        opn = mocker.patch("embedops_cli.eotools.open")
        opn.return_value = file_obj

        req_mock = mocker.patch("embedops_cli.eotools.post_dict")
        req_mock.return_value = FakeResponse()

        headers = {
            "X-API-Key": settings.api_repo_key,
        }
        data = {"type": "junit"}
        files = {
            "data": (
                report_filename,
                file_obj,
                "application/xml+junit",
            )
        }

        post_test_report(report_filename)
        req_mock.assert_called_with(
            settings.ci_artifact_endpoint,
            data_dict=data,
            file_dict=files,
            headers=headers,
        )



"""Reports the results of a CI run's test stage"""

import sys
from pprint import pformat
from logging import getLogger
from embedops_cli.config import settings
from embedops_cli.utilities import post_dict

_logger = getLogger(__name__)


def post_test_report(filename):
    """Post the contents of the test run file to the metrics API"""
    try:
        headers = {
            "X-API-Key": settings.api_repo_key,
        }
    except AttributeError as exc:
        _logger.info("No API Repo Key provided. Report will not be sent.")
        _logger.info(exc)
        sys.exit(1)

    data = {"type": "junit"}
    with open(filename, "rb") as file_obj:
        files = {
            "data": (
                filename,
                file_obj,
                "application/xml+junit",
            )
        }

        response = post_dict(
            settings.ci_artifact_endpoint,
            data_dict=data,
            file_dict=files,
            headers=headers,
        )

    if response.status_code != 200:
        _logger.error(
            (
                f"\nFAILING: Expected response type {200}(Created)"
                f"from ci run artifact endpoint, got {response.status_code}"
            )
        )
        response_string = pformat(response.json(), indent=4)
        _logger.error(response_string)
        sys.exit(1)

    response_string = pformat(response.json(), indent=4)
    _logger.info(response_string)


def main():
    """Entrypoint for artifact creation"""
    post_test_report(settings.test_output)



"""A collection of tools for woriking with YAML CI config files"""
import yaml
from embedops_cli.eo_types import BadYamlFileException


def open_yaml(yaml_filename: str):
    """Open YAML file and load it for parsing"""
    with open(yaml_filename, "r", encoding="utf-8") as stream:
        try:
            glyml = yaml.safe_load(stream)
            return glyml
        except yaml.YAMLError as err:
            raise BadYamlFileException() from err



"""
`check_api_client`
=======================================================================
Functional tests for the api client

* Author(s): Bryan Siepert
"""
#!/usr/bin/env python
from logging import getLogger
import embedops_cli.api as client
from embedops_cli.config import settings

# Configure API key authorization: ApikeyAuth
_logger = getLogger(__file__)
configuration = client.Configuration()

configuration.api_key["X-API-Key"] = settings.api_repo_key
configuration.host = f"{settings.host}/api/v1"

# create an instance of the API class
api_instance = client.ApiClient(configuration)
body = client.CIRunCreateProps(
    branch="pipline-elements",
    commit_id="49384853c0ca826f143a37b17df09eeaf01a6f63",
    type="unittest",
)

ci_run = api_instance.create_ci_run_from_ci(body)

_logger.info("created CI run:")
_logger.info(ci_run)

metric_attrs = client.MetricCreateProps(
    ci_run_id=ci_run.id,
    name="ram_size",
    value=1099,
    dimensions={"build_target": "debug"},
)
ci_run_id = ci_run.id
name = "ram_size"
value = 1099
dimensions = {"build_target": "debug"}

new_metric = api_instance.create_metric_from_ci(metric_attrs)
_logger.info("created new metric: %s", new_metric)

type = "junit"
data = "junit-metrics-on-suite-withfails.xml"
api_response = api_instance.upload_ci_run_artifact(type, data, ci_run_id)

_logger.info(api_response)
_logger.info("finishing:")
_logger.info("setting CI run status to success")

body = client.CIRunUpdateProps()  # CIRunUpdateProps | Data to update the CIRun
body.status = "success"
ci_run = api_instance.update_ci_run_from_ci(body, ci_run_id=ci_run.id)

_logger.info("\nresponse:")
_logger.info(ci_run)
del api_instance



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class CIRunCreateProps(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "branch": "str",
        "commit_id": "str",
        "name": "str",
        "pipeline_id": "str",
        "pipeline_url": "str",
        "status": "str",
        "type": "str",
    }

    attribute_map = {
        "branch": "branch",
        "commit_id": "commitId",
        "name": "name",
        "pipeline_id": "pipelineId",
        "pipeline_url": "pipelineUrl",
        "status": "status",
        "type": "type",
    }

    def __init__(
        self,
        branch=None,
        commit_id=None,
        name=None,
        pipeline_id=None,
        pipeline_url=None,
        status=None,
        type=None,
    ):  # noqa: E501
        """CIRunCreateProps - a model defined in Swagger"""  # noqa: E501
        self._branch = None
        self._commit_id = None
        self._name = None
        self._pipeline_id = None
        self._pipeline_url = None
        self._status = None
        self._type = None
        self.discriminator = None
        self.branch = branch
        self.commit_id = commit_id
        if name is not None:
            self.name = name
        if pipeline_id is not None:
            self.pipeline_id = pipeline_id
        if pipeline_url is not None:
            self.pipeline_url = pipeline_url
        if status is not None:
            self.status = status
        self.type = type

    @property
    def branch(self):
        """Gets the branch of this CIRunCreateProps.  # noqa: E501


        :return: The branch of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._branch

    @branch.setter
    def branch(self, branch):
        """Sets the branch of this CIRunCreateProps.


        :param branch: The branch of this CIRunCreateProps.  # noqa: E501
        :type: str
        """
        if branch is None:
            raise ValueError(
                "Invalid value for `branch`, must not be `None`"
            )  # noqa: E501

        self._branch = branch

    @property
    def commit_id(self):
        """Gets the commit_id of this CIRunCreateProps.  # noqa: E501


        :return: The commit_id of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._commit_id

    @commit_id.setter
    def commit_id(self, commit_id):
        """Sets the commit_id of this CIRunCreateProps.


        :param commit_id: The commit_id of this CIRunCreateProps.  # noqa: E501
        :type: str
        """
        if commit_id is None:
            raise ValueError(
                "Invalid value for `commit_id`, must not be `None`"
            )  # noqa: E501

        self._commit_id = commit_id

    @property
    def name(self):
        """Gets the name of this CIRunCreateProps.  # noqa: E501


        :return: The name of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this CIRunCreateProps.


        :param name: The name of this CIRunCreateProps.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def pipeline_id(self):
        """Gets the pipeline_id of this CIRunCreateProps.  # noqa: E501


        :return: The pipeline_id of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._pipeline_id

    @pipeline_id.setter
    def pipeline_id(self, pipeline_id):
        """Sets the pipeline_id of this CIRunCreateProps.


        :param pipeline_id: The pipeline_id of this CIRunCreateProps.  # noqa: E501
        :type: str
        """

        self._pipeline_id = pipeline_id

    @property
    def pipeline_url(self):
        """Gets the pipeline_url of this CIRunCreateProps.  # noqa: E501


        :return: The pipeline_url of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._pipeline_url

    @pipeline_url.setter
    def pipeline_url(self, pipeline_url):
        """Sets the pipeline_url of this CIRunCreateProps.


        :param pipeline_url: The pipeline_url of this CIRunCreateProps.  # noqa: E501
        :type: str
        """

        self._pipeline_url = pipeline_url

    @property
    def status(self):
        """Gets the status of this CIRunCreateProps.  # noqa: E501


        :return: The status of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this CIRunCreateProps.


        :param status: The status of this CIRunCreateProps.  # noqa: E501
        :type: str
        """
        allowed_values = ["running", "success", "failure"]  # noqa: E501
        if status not in allowed_values:
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                    status, allowed_values
                )
            )

        self._status = status

    @property
    def type(self):
        """Gets the type of this CIRunCreateProps.  # noqa: E501


        :return: The type of this CIRunCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this CIRunCreateProps.


        :param type: The type of this CIRunCreateProps.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError(
                "Invalid value for `type`, must not be `None`"
            )  # noqa: E501
        allowed_values = [
            "cppcheck",
            "unittest",
            "codecoverage",
            "memusage",
        ]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}".format(  # noqa: E501
                    type, allowed_values
                )
            )

        self._type = type

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(CIRunCreateProps, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CIRunCreateProps):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
`eo_types`
=======================================================================
Module will hold the enum classes for EmbedOps Tools
* Author(s): Bailey Steinfadt
"""

from enum import Enum

GH_CI_CONFIG_FILE_PATH = ".github/workflows"
BB_CI_CONFIG_FILENAME = "bitbucket-pipelines.yml"
GL_CI_CONFIG_FILENAME = ".gitlab-ci.yml"
EO_CI_CONFIG_FILENAME = ".embedops-ci.yml"
SUPPORTED_CI_FILENAMES = [
    BB_CI_CONFIG_FILENAME,
    GL_CI_CONFIG_FILENAME,
    EO_CI_CONFIG_FILENAME,
]


class YamlType(Enum):
    """Types of Yaml Files EmbedOps Tools supports"""

    UNSUPPORTED = 0
    GITLAB = 1
    BITBUCKET = 2
    GITHUB = 3


class LocalRunContext:
    """Object to store the context for locally run CI jobs"""

    def __init__(
        self,
        job_name: str,
        docker_tag: str,
        script_lines: list = None,
        variables: dict = None,
    ):
        self._job_name = job_name.strip('"')
        self._docker_tag = docker_tag.strip('"')
        if script_lines is None:
            self._script = []
        else:
            self._script = script_lines
        if variables is None:
            self._variables = {}
        else:
            self._variables = variables

    @property
    def job_name(self):
        """String with the name of the job"""
        return self._job_name

    @property
    def docker_tag(self):
        """String for the Docker tag the job will be launched in"""
        return self._docker_tag

    @property
    def script(self):
        """List containing the job's script from the YAML file, if it exists"""
        return self._script

    @property
    def variables(self):
        """Dictionary with any variables defined in the YAML file"""
        return self._variables


##################################################################################################
########################################### EXCEPTIONS ###########################################
##################################################################################################


class EmbedOpsException(Exception):
    """Base class for all EmbedOps exceptions"""

    def __init__(
        self, message="EmbedOps encountered an internal error", fix_message=""
    ):
        self.message = message
        self.fix_message = fix_message
        super().__init__(self.message)


############################################## YAML ##############################################


class UnsupportedYamlTypeException(EmbedOpsException):
    """Raised when an Unsupported YAML type is input"""

    ERROR_MSG = "CI configuration YAML file is not one of the supported filenames\n"
    ERROR_FIX = (
        "Make sure one of the following CI configuration files is in the current directory:\n"
        "    " + "\n    ".join(SUPPORTED_CI_FILENAMES) + "\n"
        "\nNOTE: The GitHub CI configuration file should be in the .github/workflows directory"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class NoYamlFileException(EmbedOpsException):
    """Raised when no YAML file is found"""

    ERROR_MSG = "CI configuration YAML file could not be found\n"
    ERROR_FIX = (
        "Either specify a filename with the --filename option or make sure\n"
        "one of the following CI configuration files is in the current directory:\n"
        "    " + "\n    ".join(SUPPORTED_CI_FILENAMES) + "\n"
        "\nNOTE: The GitHub CI configuration file should be in the .github/workflows directory"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class BadYamlFileException(EmbedOpsException):
    """Raised when a bad YAML file is found"""

    ERROR_MSG = "CI configuration YAML file could not be parsed\n"
    ERROR_FIX = (
        "Check your YAML for syntax errors. \n"
        "Email support@embedops.io if you have questions."
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class MultipleYamlFilesException(EmbedOpsException):
    """Raised when multiple YAML files are found"""

    ERROR_MSG = "Multiple CI configuration files were found.\n"
    ERROR_FIX = (
        "Please specify the desired CI configuration file by using the --filename flag.\n\n"
        "Syntax: embedops-cli jobs --filename <PATH_TO_CI_CONFIG_FILE> run <JOB_NAME>"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


######################################### Authorization ##########################################


class UnauthorizedUserException(EmbedOpsException):
    """Raised when there is no Authorization Token found in the user's secrets file"""

    ERROR_MSG = "No EmbedOps credentials found\n"

    ERROR_FIX = (
        "EmbedOps CLI is even better with an account!\n"
        "If you do not have an account and would like to learn more about EmbedOps, "
        "contact support@embedops.io"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class LoginFailureException(EmbedOpsException):
    """Raised when logging into the Embedops backend fails"""

    ERROR_MSG = "A problem was encountered while logging into EmbedOps.\n"

    ERROR_FIX = (
        "Check your credentials on app.embedops.io and try again.\n"
        "If you encounter further issues, please contact support:\n"
        "support@embedops.io"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


############################################# Docker #############################################


class NoDockerContainerException(EmbedOpsException):
    """Raised when no Docker container is found in the CI configuration file"""

    ERROR_MSG = (
        "Docker container is not found in the job or in the CI configuration file.\n"
    )
    ERROR_FIX = (
        "A Docker container must be provided to run a job.\n\n"
        "For GitLab CI, use the `image` keyword.\n"
        "It can be used as part of a job, in the `default` section, or globally.\n\n"
        "For GitHub CI, use the `container` keyword.\n"
        "It can only be used as part of a job."
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class InvalidDockerContainerException(EmbedOpsException):
    """Raised when an invalid Docker container is detected"""

    ERROR_MSG = "Docker container is invalid.\n"
    ERROR_FIX = (
        "If your Docker container is hosted on a private registry,\n"
        "do not include http:// in your Docker container link."
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class DockerNotRunningException(EmbedOpsException):
    """Raised when the Docker daemon is not running"""

    ERROR_MSG = "Docker is not running\n"
    ERROR_FIX = (
        "Start or restart Docker desktop. \n"
        "Look for the whale logo in your system status tray\n"
        'and check that it says "Docker Desktop running"'
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class DockerRegistryException(EmbedOpsException):
    """Raised when a problem accessing the registry is encountered"""

    ERROR_MSG = (
        "We were unable to authenticate with the package registry, "
        "or the image name is not correct.\n"
    )
    ERROR_FIX = (
        "To access the required Docker images needed run this job,\n"
        "login to the EmbedOps Docker registry using this command:\n"
        "\n"
        "    docker login registry.embedops.com\n"
        "\n"
        "When prompted, login with your access token, found on app.embedops.io\n"
        "Check in registry that the image name exists."
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)


class UnknownDockerException(EmbedOpsException):
    """Raised when an error with Docker is encountered that we haven't otherwise handled"""

    ERROR_MSG = (
        "No clue what happened, but Docker didn't run.\n  Here there be dragons. \n"
    )
    ERROR_FIX = (
        "Turn everything off and on again.\n"
        "Then, if it's still broken, file a bug report with Dojo Five:\n"
        "support@embedops.io"
    )

    def __init__(
        self,
        message=ERROR_MSG,
        fix_message=ERROR_FIX,
    ):
        super().__init__(message, fix_message)



"""
`gh_parser`
=======================================================================
Parser to pull job contexts from .github-ci.yml files
* Author(s): Zhi Xuen Lai
"""
import logging
import re
from embedops_cli.yaml_tools import open_yaml
from ..eo_types import (
    BadYamlFileException,
    LocalRunContext,
)

_logger = logging.getLogger(__name__)


def _check_job_name(string):
    """
    Check if the job name starts with a letter or _ and
    contains only alphanumeric characters, -, or _.
    """
    # Check if the job name starts with a letter or _
    is_first_char_valid = bool(re.match("^[a-zA-Z_]", string[:1]))
    # Check if the job name contains only alphanumeric characters, -, or _
    is_whole_job_name_valid = bool(re.match("^[a-zA-Z0-9_-]*$", string))
    return is_first_char_valid and is_whole_job_name_valid


def get_job_name_list(ghyml_filename: str) -> list:
    """Get list of job names from the given YAML object"""

    try:
        ghyml = open_yaml(ghyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_name_list = []

    try:
        for item in ghyml["jobs"]:
            if not _check_job_name(item):
                raise BadYamlFileException()
            job_name_list.append(item)

        return job_name_list
    except (KeyError) as err:
        raise BadYamlFileException() from err


def get_job_list(glyml_filename: str) -> list:
    """Return the list of LocalRunContexts found in the given yaml object"""

    try:
        glyml = open_yaml(glyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_list = []

    try:
        default_var_dict = {}
        for tag, value in glyml.items():
            # update default_var_dict
            if tag == "env":
                default_var_dict.update(value)

            if tag == "jobs":
                for job, context in value.items():
                    if not _check_job_name(job):
                        raise BadYamlFileException()
                    job_ctx = _parse_job_context(job, context, default_var_dict)

                    job_list.append(job_ctx)

        return job_list
    except (TypeError, KeyError) as err:
        raise BadYamlFileException() from err


def _parse_job_context(yaml_tag, yaml_value, default_var_dict):
    """Return a LocalRunContexts by parsing a job context"""
    image = ""

    if "container" in yaml_value:
        if "image" in yaml_value["container"]:
            image = yaml_value["container"]["image"]

    # Does not necessarily have variables
    # if it does, add them (via update()) to the default_var_dict
    var_dict = {}
    var_dict.update(default_var_dict)
    if "env" in yaml_value:
        var_dict.update(yaml_value["env"])

    # TODO: parse the | token appropriately
    #       (Note: it's able to parse simple multi-line command)
    # TODO: handle `uses` keyword
    #       (Note: only `run` and `env` keywords are handled at the moment)
    script_list = []
    if "steps" in yaml_value:
        for line in yaml_value["steps"]:
            if "run" in line:
                script_list.append(line["run"])
            if "env" in line:
                var_dict.update(line["env"])
    else:
        raise BadYamlFileException()

    return LocalRunContext(yaml_tag, image, script_list, var_dict)



"""
`embedops_cli`
=======================================================================
* Author(s): Bailey Steinfadt, Zhi Xuen Lai
"""
from .version import (
    __version__,
)  # Import __version__ attribute so that it is visible to the setup.cfg



#!/usr/bin/env python
"""
`create_run.py`
=======================================================================
A small script to create an EmbedOps CI run record and return the new ID
* Author(s): Bryan Siepert
"""
from embedops_cli.eotools.ci_run import CIRun

if __name__ == "__main__":
    ci_run = CIRun()
    ci_run.main()



#!/usr/bin/env python
"""
`parse.py`
=======================================================================
A script to parse information from build logs
* Author(s): Bryan Siepert
"""
import re
from sys import exit as sys_exit
from os.path import exists as file_exists
from pprint import pformat
import logging

from embedops_cli.config import settings
from embedops_cli.utilities import post_dict, get_compiler
from embedops_cli.eotools.log_parser.iar import SIZE_PATTERN as IAR_SIZE_PATTERN
from embedops_cli.eotools.log_parser.gnu_size_berkeley import (
    SIZE_PATTERN as SIZE_SIZE_PATTERN,
)

_logger = logging.getLogger(__name__)

EXPECTED_RETURN_CODE = 200


def parse_storage_sizes(log_filename, build_size_regex):
    """Check each line in a log file for compiler RAM, flash data, and flash code sizes"""

    build_data = None
    storage_sizes = None
    with open(log_filename, "r", encoding="ascii") as build_log:
        build_data = build_log.read()

    build_results = re.search(build_size_regex, build_data)

    if build_results is None:
        return storage_sizes

    storage_sizes = {
        "flash_code_size": None,
        "flash_data_size": None,
        "ram_size": None,
    }
    storage_sizes["ram_size"] = int(build_results["ram_size"].replace("'", ""))
    storage_sizes["flash_code_size"] = int(
        build_results["flash_code_size"].replace("'", "")
    )
    storage_sizes["flash_data_size"] = int(
        build_results["flash_data_size"].replace("'", "")
    )
    storage_sizes["dimensions"] = {
        "build_target": build_results["build_output_filename"].split("/")[-1]
    }

    # a better solution would be to look for the -g option but it's more complicated
    return storage_sizes


def _report_metrics(build_metrics, run_id, embedops_repo_key):

    headers = {"X-API-Key": embedops_repo_key, "Content-Type": "application/json"}
    if run_id == "LOCAL":
        _logger.info("\nResults:")

    for key in build_metrics:
        if key == "dimensions":
            continue
        stats_data = {
            "ciRunId": run_id,
            "name": key,
            "value": build_metrics[key],
            "dimensions": build_metrics["dimensions"],
        }

        if run_id == "LOCAL":
            _logger.info(f"\t{key} : {pformat(build_metrics[key])}")
        else:
            response = post_dict(
                settings.metrics_endpoint,
                json_dict=stats_data,
                headers=headers,
            )
            # TODO: Refactor this to remove the duplication with similar code in `create_run.py`,
            # perhaps in a shared API library :O
            if response.status_code != EXPECTED_RETURN_CODE:
                _logger.error(
                    f"FAILING: Expected response type {EXPECTED_RETURN_CODE}(Created)"
                    f"from metrics creation endpoint, got {response.status_code}"
                )
                response_string = pformat(response.json(), indent=4)
                _logger.error(response_string)

                sys_exit(1)
            response_string = pformat(response.json(), indent=4)
            _logger.info("Created metric:")
            _logger.info(response_string)


def parse_reports(input_filename, parsing_regex, run_id, embedops_repo_key):
    """Parse the given file for compile sized totals"""
    if not file_exists(input_filename):
        raise FileNotFoundError(
            f"The given input file {input_filename} cannot be found"
        )
    storage_sizes = parse_storage_sizes(input_filename, parsing_regex)
    _logger.info(f"Got storage sizes: {storage_sizes}")
    if storage_sizes is None:
        _logger.error("no build target sizes found")
        sys_exit(1)
    _report_metrics(storage_sizes, run_id, embedops_repo_key)


def main():
    """The main entrypoint for the module, to allow for binary-izing"""
    compiler = get_compiler()
    if compiler == "IAR":
        size_regex = IAR_SIZE_PATTERN
        _logger.info("IAR Pattern loaded")
    elif compiler in ("TI", "GCC"):
        size_regex = SIZE_SIZE_PATTERN
        _logger.info("gnu size pattern loaded for TI and GCC")
    else:
        _logger.warning("EMBEDOPS_COMPILER not set")
        sys_exit(1)
    input_file = settings.input_file
    run_id = settings.run_id

    try:
        api_repo_key = settings.api_repo_key
    except AttributeError:
        api_repo_key = None

    _logger.info(f"EMBEDOPS_INPUT_FILE {input_file}")
    _logger.info(f"EMBEDOPS_RUN_ID {run_id}")

    if run_id is None:
        _logger.warning(
            "EMBEDOPS_RUN_ID not set. Assuming local build, will not push metrics"
        )
        run_id = "LOCAL"
    elif run_id == "LOCAL":
        _logger.info("Local build requested. Will not push metrics")
    elif api_repo_key is None:
        _logger.warning(
            "EMBEDOPS_API_REPO_KEY not set. Assuming local build, will not push metrics."
        )
        run_id = "LOCAL"

    _logger.info("EMBEDOPS_API_REPO_KEY set (not echoing)")

    # this should read directly from settings
    parse_reports(input_file, size_regex, run_id, api_repo_key)


if __name__ == "__main__":
    main()



"""
`build_log_parser_test`
=======================================================================
Unit tests for parsing of build logs generated by compilers
* Author(s): Jimmy Gomez
"""
import os

from embedops_cli.eotools.log_parser.iar import SIZE_PATTERN as IAR_SIZE_PATTERN
from embedops_cli.eotools.log_parser.gnu_size_berkeley import (
    SIZE_PATTERN as GCC_SIZE_PATTERN,
)
from embedops_cli.eotools.parse import parse_storage_sizes


def test_parsing_of_iar_build_log():
    """Test parsing of build log generated by IAR"""
    parsing_regex = IAR_SIZE_PATTERN
    file_dir = os.path.dirname(os.path.realpath(__file__))
    input_filename = file_dir + "/iar_example_build.log"
    storage_sizes = parse_storage_sizes(input_filename, parsing_regex)
    assert storage_sizes["dimensions"]["build_target"] == "componentA.out"
    assert storage_sizes["flash_code_size"] == 7636
    assert storage_sizes["flash_data_size"] == 136
    assert storage_sizes["ram_size"] == 1036


def test_parsing_of_gcc_build_log():
    """Test parsing of build log generated by GCC"""
    parsing_regex = GCC_SIZE_PATTERN
    file_dir = os.path.dirname(os.path.realpath(__file__))
    input_filename = file_dir + "/gcc_example_build.log"
    storage_sizes = parse_storage_sizes(input_filename, parsing_regex)
    assert storage_sizes["flash_code_size"] == 36080
    assert storage_sizes["flash_data_size"] == 664
    assert storage_sizes["ram_size"] == 50960
    assert storage_sizes["dimensions"]["build_target"] == "application.elf"



"""
`api_interface_test`
=======================================================================
Unit tests for code to upload XML files to the API
* Author(s): Bryan Siepert
"""
import pytest
from dynaconf import Dynaconf

DEFAULT_JOB_URL = "https://gitlab.com/luminostics/corefw/-/jobs/1890073938"


@pytest.fixture(scope="function", autouse=True)
def set_env_like_ci(monkeypatch):
    """Makes the environment look like it does in a CI system"""
    monkeypatch.setenv("CI_JOB_URL", DEFAULT_JOB_URL)
    monkeypatch.setenv("CI_COMMIT_SHA", "d34db33f")
    monkeypatch.setenv("CI_COMMIT_REF_NAME", "main")


def set_test_settings():
    """A fixture to setup Dynaconf to use the testing environment"""
    return Dynaconf(
        FORCE_ENV_FOR_DYNACONF="loader",
        loaders=["embedops_cli.ci_config_loader"],
        envvar_prefix="EMBEDOPS",
        environments=True,
        settings_files=["settings.toml", ".secrets.toml"],
    )


def test_bb_get_job_url(monkeypatch):
    """Test that the job url for bitbucket is returned correctly"""
    monkeypatch.setenv("BITBUCKET_COMMIT", "c48aa")
    monkeypatch.setenv("BITBUCKET_BRANCH", "main")
    monkeypatch.setenv(
        "BITBUCKET_GIT_HTTP_ORIGIN", "https://bitbucket.org/baileysteinfadt/ti-testbed"
    )
    monkeypatch.setenv("BITBUCKET_STEP_UUID", "{4b85eb71-8130-4234-b57e-b7afe874eafc}")
    monkeypatch.setenv("BITBUCKET_BUILD_NUMBER", "76")

    settings = set_test_settings()

    assert settings.job_url == (
        f"https://bitbucket.org/baileysteinfadt/ti-testbed/addon/"
        f"pipelines/home#!/results/76/steps/{{4b85eb71-8130-4234-b57e-b7afe874eafc}}"
    )

    monkeypatch.setenv(
        "BITBUCKET_GIT_HTTP_ORIGIN", "https://bitbucket.org/baileysteinfadt/ti-testbed"
    )
    monkeypatch.setenv("BITBUCKET_STEP_UUID", "{not-a-real-uuid}")
    monkeypatch.setenv("BITBUCKET_BUILD_NUMBER", "2112")

    settings2 = set_test_settings()
    assert settings2.job_url == (
        f"https://bitbucket.org/baileysteinfadt/ti-testbed/addon/"
        f"pipelines/home#!/results/2112/steps/{{not-a-real-uuid}}"
    )


def test_gl_get_job_url(monkeypatch):
    """Test that the job url for gitlab is returned correctly"""
    settings = set_test_settings()

    assert settings.job_url == DEFAULT_JOB_URL


def test_no_ci_url_env(monkeypatch):
    """Test that the job url is not set it is not returned"""
    monkeypatch.delenv("CI_COMMIT_SHA")
    monkeypatch.delenv("CI_COMMIT_REF_NAME")
    monkeypatch.delenv("CI_JOB_URL")

    settings = set_test_settings()

    assert settings.get("job_url", None) is None



"""
`bb_parser_test`
=======================================================================
Unit tests for the parser to pull job contexts from bitbucket-pipelines.yml files
* Author(s): Bailey Steinfadt
"""
import pytest

from embedops_cli.yaml_tools import bb_parser, yaml_utilities
from embedops_cli.eo_types import (
    BadYamlFileException,
)
from tests import BBYML_FILENAME

BBYML_UNNAMED_STEP_FILENAME = "tests/test-unnamed-step-bitbucket-pipelines.yml"
BBYML_MISSING_STEP_NAME_VALUE_FILENAME = (
    "tests/test-missing-step-name-value-bitbucket-pipelines.yml"
)
BBYML_MISSING_IMAGE_NAME_VALUE_FILENAME = (
    "tests/test-missing-image-name-value-bitbucket-pipelines.yml"
)
BBYML_MISSING_SCRIPT_VALUE_FILENAME = (
    "tests/test-missing-script-value-bitbucket-pipelines.yml"
)


def test_get_job_list_with_unnamed_step_exception():
    """test that exception is raised for job with unnamed step"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_list(BBYML_UNNAMED_STEP_FILENAME)


def test_get_job_name_list_with_unnamed_step_exception():
    """test that exception is raised for job with unnamed step"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_name_list(BBYML_UNNAMED_STEP_FILENAME)


def test_get_job_list_with_missing_step_name_value_exception():
    """test that exception is raised for job with missing step name value"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_list(BBYML_MISSING_STEP_NAME_VALUE_FILENAME)


def test_get_job_name_list_with_missing_step_name_value_exception():
    """test that exception is raised for job with missing step name value"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_name_list(BBYML_MISSING_STEP_NAME_VALUE_FILENAME)


def test_get_job_list_with_missing_image_name_value_exception():
    """test that exception is raised for job with missing image name value"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_list(BBYML_MISSING_IMAGE_NAME_VALUE_FILENAME)


def test_get_job_list_with_missing_script_value_exception():
    """test that exception is raised for job with missing script value"""
    with pytest.raises(BadYamlFileException):
        assert bb_parser.get_job_list(BBYML_MISSING_SCRIPT_VALUE_FILENAME)


def test_get_job_name_list():
    """Test retrieving the list of job names from the YAML"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    assert len(job_name_list) == 13
    assert job_name_list[0] == "StepScript"
    assert job_name_list[5] == "ParallelNoImage"


def test_get_job_list():
    """Test retrieving the list of complete local run context from the YAML"""
    job_list = bb_parser.get_job_list(BBYML_FILENAME)
    assert len(job_list) == 13
    assert job_list[0].job_name == "StepScript"
    assert len(job_list[0].docker_tag) > 0
    assert len(job_list[0].script) > 0
    assert job_list[5].job_name == "ParallelNoImage"
    assert len(job_list[5].docker_tag) == 0
    assert len(job_list[5].script) > 0


def test_get_job_list_default_image():
    """Test retrieving the list of complete local run context from the YAML
    where the step uses the default image"""
    job_list = bb_parser.get_job_list(BBYML_FILENAME)
    job = [job for job in job_list if job.job_name == "StepNoImage"][0]
    assert job.job_name == "StepNoImage"
    assert len(job.docker_tag) > 0
    assert len(job.script) > 0


def test_get_job_list_no_script():
    """Test retrieving the list of complete local run context from the YAML
    when the step has no script"""
    job_list = bb_parser.get_job_list(BBYML_FILENAME)
    job = [job for job in job_list if job.job_name == "StepNoScript"][0]
    assert job.job_name == "StepNoScript"
    assert len(job.docker_tag) > 0
    assert len(job.script) == 0


def test_get_job_context():
    """Test getting the job context for an indicated job"""
    requested_name = "StepScript"
    job = yaml_utilities.get_job_context_for_name(
        bb_parser, BBYML_FILENAME, requested_name
    )
    assert job.job_name == requested_name

    requested_name_parallel = "ParallelScript"
    job = yaml_utilities.get_job_context_for_name(
        bb_parser, BBYML_FILENAME, requested_name_parallel
    )
    assert job.job_name == requested_name_parallel


def test_step_with_literal_multiline_block():
    """Test retrieving a literal multiline block"""
    job = yaml_utilities.get_job_context_for_name(
        bb_parser, BBYML_FILENAME, "StepWithLiteralMultilineBlock"
    )
    assert (
        job.script[0]
        == 'FILE=.clang-format\nif [ -f "$FILE" ]; then echo "$FILE exists. Use repository $FILE."; else echo "$FILE does not exist. Use container $FILE."; cp /tools/.clang-format .clang-format; fi\n'
    )


def test_step_with_folded_multiline_block():
    """Test retrieving a folded multiline block"""
    job = yaml_utilities.get_job_context_for_name(
        bb_parser, BBYML_FILENAME, "StepWithFoldedMultilineBlock"
    )
    assert (
        job.script[0]
        == 'FILE=.clang-format\nif [ -f "$FILE" ]; then echo "$FILE exists. Use repository $FILE."; else echo "$FILE does not exist. Use container $FILE."; cp /tools/.clang-format .clang-format; fi\n'
    )



"""
`yaml_utilities`
=======================================================================
Shared YAML parsing utilities
* Author(s): Bailey Steinfadt and Bryan Siepert
"""

from logging import getLogger
from os import listdir, path
from embedops_cli.yaml_tools import bb_parser, gl_parser, gh_parser

from embedops_cli.eo_types import (
    BadYamlFileException,
    NoYamlFileException,
    UnsupportedYamlTypeException,
    MultipleYamlFilesException,
    LocalRunContext,
    SUPPORTED_CI_FILENAMES,
    GH_CI_CONFIG_FILE_PATH,
    GL_CI_CONFIG_FILENAME,
    BB_CI_CONFIG_FILENAME,
    EO_CI_CONFIG_FILENAME,
)

_logger = getLogger(__name__)


def get_job_context_for_name(
    parser, yml_filename: str, requested_name: str
) -> LocalRunContext:
    """Get job context for requested job name from specified yml object"""

    try:
        for job in parser.get_job_list(yml_filename):
            if job.job_name == requested_name:
                return job
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    # if job was not found, return None
    return None


def get_yaml_in_directory(directory="."):
    """Looks for supported YAML files in the indicated directory"""
    current_dir_files = listdir(directory)
    for dir_file in SUPPORTED_CI_FILENAMES:
        if dir_file in current_dir_files:
            filename = dir_file

            _logger.debug(f"Found YAML: {filename} in directory: {directory}")
            break
    # No CI configuration file is found in the root directory
    # Check for the GitHub CI configuration file in GH_CI_CONFIG_FILE_PATH
    else:
        if not path.isdir(GH_CI_CONFIG_FILE_PATH):
            raise NoYamlFileException()

        files = listdir(GH_CI_CONFIG_FILE_PATH)
        if len(files) != 1:
            raise MultipleYamlFilesException()

        if files[0].lower().endswith(".yaml") or files[0].endswith(".yml"):
            filename = path.join(GH_CI_CONFIG_FILE_PATH, files[0])
            _logger.debug(
                f"\nFound YAML: {files[0]} in directory: {GH_CI_CONFIG_FILE_PATH} \n"
            )
        else:
            raise NoYamlFileException()
    return filename


def get_job_list(filename: str) -> list:
    """Retrieve the job list from the given file using the required parser"""
    try:
        parser = get_correct_parser_type(filename)
    except UnsupportedYamlTypeException as exc:
        raise UnsupportedYamlTypeException() from exc

    try:
        job_list = parser.get_job_list(filename)
        return job_list
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc


def get_correct_parser_type(filename: str):
    """Get the parser for a given filename"""
    normalized_filename = filename.lower()
    if GL_CI_CONFIG_FILENAME in normalized_filename:
        return gl_parser
    if EO_CI_CONFIG_FILENAME in normalized_filename:
        return gl_parser
    if BB_CI_CONFIG_FILENAME in normalized_filename:
        return bb_parser
    # The name of the GitHub CI configuration file is not fixed
    # but the file must be in the GH_CI_CONFIG_FILE_PATH
    if GH_CI_CONFIG_FILE_PATH in normalized_filename:
        return gh_parser
    raise UnsupportedYamlTypeException()



"""
`environment_utilities_test`
=======================================================================
Unit tests for the module that manages environment context for the CLI
* Author(s): Bailey Steinfadt
"""
import os
from embedops_cli import environment_utilities as envutil


def test_create_and_delete_job_env_file():
    """
    Test that a file is made with no errors.
    And that it's gone after deletion.
    """
    # create file
    testdict = {"key": "value"}
    envutil.create_job_env_file(testdict)

    assert os.path.exists(envutil.JOB_ENV_FILE)

    # Check the file contains what we think
    contents = ""
    with open(envutil.JOB_ENV_FILE, encoding="utf-8") as env_file:
        contents = env_file.read()
    assert "key=value" in contents

    # delete file
    envutil.delete_job_env_file()

    assert os.path.exists(envutil.JOB_ENV_FILE) is False


def test_clean_env_file():
    """
    Test that the scrubbing fuction works
    to put data into an env file for dockr use
    """

    dirty_env_contents = [
        "# Comments that should not be copied   \n",
        '# COMMENTED_ENV_VAR="should not be copied"\n',
        "ENV_NO_QUOTES=unchanged\n",
        'ENV_WITH_QUOTES="now_no_quotes"\n',
        "\n",  # Empty line
        "ENV_WITH_SINGLE_QUOTES='now_no_quotes'\n",
    ]

    clean_env_contents = envutil._clean_env_file(dirty_env_contents)

    # Check that the "bad" lines got removed
    assert len(clean_env_contents) == 3
    assert dirty_env_contents[0] not in clean_env_contents
    assert dirty_env_contents[1] not in clean_env_contents
    assert dirty_env_contents[4] not in clean_env_contents

    # check that the good lines were unchanged or just had two chars removed
    assert dirty_env_contents[2] in clean_env_contents
    assert len(dirty_env_contents[3]) - 2 == len(clean_env_contents[1])
    assert len(dirty_env_contents[5]) - 2 == len(clean_env_contents[2])



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class User(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "onboarding_dismissed": "bool",
        "bitbucket_code": "str",
        "auth0_id": "str",
        "created_at": "datetime",
        "email": "str",
        "family_name": "str",
        "given_name": "str",
        "groups": "list[UserGroupProps]",
        "id": "str",
        "name": "str",
        "nickname": "str",
        "invite_status": "str",
        "invite_ticket": "str",
        "updated_at": "datetime",
    }

    attribute_map = {
        "onboarding_dismissed": "onboardingDismissed",
        "bitbucket_code": "bitbucketCode",
        "auth0_id": "auth0Id",
        "created_at": "createdAt",
        "email": "email",
        "family_name": "familyName",
        "given_name": "givenName",
        "groups": "groups",
        "id": "id",
        "name": "name",
        "nickname": "nickname",
        "invite_status": "inviteStatus",
        "invite_ticket": "inviteTicket",
        "updated_at": "updatedAt",
    }

    def __init__(
        self,
        onboarding_dismissed=None,
        bitbucket_code=None,
        auth0_id=None,
        created_at=None,
        email=None,
        family_name=None,
        given_name=None,
        groups=None,
        id=None,
        name=None,
        nickname=None,
        invite_status=None,
        invite_ticket=None,
        updated_at=None,
    ):  # noqa: E501
        """User - a model defined in Swagger"""  # noqa: E501
        self._onboarding_dismissed = None
        self._bitbucket_code = None
        self._auth0_id = None
        self._created_at = None
        self._email = None
        self._family_name = None
        self._given_name = None
        self._groups = None
        self._id = None
        self._name = None
        self._nickname = None
        self._invite_status = None
        self._invite_ticket = None
        self._updated_at = None
        self.discriminator = None
        if onboarding_dismissed is not None:
            self.onboarding_dismissed = onboarding_dismissed
        if bitbucket_code is not None:
            self.bitbucket_code = bitbucket_code
        if auth0_id is not None:
            self.auth0_id = auth0_id
        if created_at is not None:
            self.created_at = created_at
        if email is not None:
            self.email = email
        if family_name is not None:
            self.family_name = family_name
        if given_name is not None:
            self.given_name = given_name
        if groups is not None:
            self.groups = groups
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if nickname is not None:
            self.nickname = nickname
        if invite_status is not None:
            self.invite_status = invite_status
        if invite_ticket is not None:
            self.invite_ticket = invite_ticket
        if updated_at is not None:
            self.updated_at = updated_at

    @property
    def onboarding_dismissed(self):
        """Gets the onboarding_dismissed of this User.  # noqa: E501


        :return: The onboarding_dismissed of this User.  # noqa: E501
        :rtype: bool
        """
        return self._onboarding_dismissed

    @onboarding_dismissed.setter
    def onboarding_dismissed(self, onboarding_dismissed):
        """Sets the onboarding_dismissed of this User.


        :param onboarding_dismissed: The onboarding_dismissed of this User.  # noqa: E501
        :type: bool
        """

        self._onboarding_dismissed = onboarding_dismissed

    @property
    def bitbucket_code(self):
        """Gets the bitbucket_code of this User.  # noqa: E501


        :return: The bitbucket_code of this User.  # noqa: E501
        :rtype: str
        """
        return self._bitbucket_code

    @bitbucket_code.setter
    def bitbucket_code(self, bitbucket_code):
        """Sets the bitbucket_code of this User.


        :param bitbucket_code: The bitbucket_code of this User.  # noqa: E501
        :type: str
        """

        self._bitbucket_code = bitbucket_code

    @property
    def auth0_id(self):
        """Gets the auth0_id of this User.  # noqa: E501


        :return: The auth0_id of this User.  # noqa: E501
        :rtype: str
        """
        return self._auth0_id

    @auth0_id.setter
    def auth0_id(self, auth0_id):
        """Sets the auth0_id of this User.


        :param auth0_id: The auth0_id of this User.  # noqa: E501
        :type: str
        """

        self._auth0_id = auth0_id

    @property
    def created_at(self):
        """Gets the created_at of this User.  # noqa: E501


        :return: The created_at of this User.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this User.


        :param created_at: The created_at of this User.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def email(self):
        """Gets the email of this User.  # noqa: E501


        :return: The email of this User.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this User.


        :param email: The email of this User.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def family_name(self):
        """Gets the family_name of this User.  # noqa: E501


        :return: The family_name of this User.  # noqa: E501
        :rtype: str
        """
        return self._family_name

    @family_name.setter
    def family_name(self, family_name):
        """Sets the family_name of this User.


        :param family_name: The family_name of this User.  # noqa: E501
        :type: str
        """

        self._family_name = family_name

    @property
    def given_name(self):
        """Gets the given_name of this User.  # noqa: E501


        :return: The given_name of this User.  # noqa: E501
        :rtype: str
        """
        return self._given_name

    @given_name.setter
    def given_name(self, given_name):
        """Sets the given_name of this User.


        :param given_name: The given_name of this User.  # noqa: E501
        :type: str
        """

        self._given_name = given_name

    @property
    def groups(self):
        """Gets the groups of this User.  # noqa: E501


        :return: The groups of this User.  # noqa: E501
        :rtype: list[UserGroupProps]
        """
        return self._groups

    @groups.setter
    def groups(self, groups):
        """Sets the groups of this User.


        :param groups: The groups of this User.  # noqa: E501
        :type: list[UserGroupProps]
        """

        self._groups = groups

    @property
    def id(self):
        """Gets the id of this User.  # noqa: E501


        :return: The id of this User.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this User.


        :param id: The id of this User.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this User.  # noqa: E501


        :return: The name of this User.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this User.


        :param name: The name of this User.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def nickname(self):
        """Gets the nickname of this User.  # noqa: E501


        :return: The nickname of this User.  # noqa: E501
        :rtype: str
        """
        return self._nickname

    @nickname.setter
    def nickname(self, nickname):
        """Sets the nickname of this User.


        :param nickname: The nickname of this User.  # noqa: E501
        :type: str
        """

        self._nickname = nickname

    @property
    def invite_status(self):
        """Gets the invite_status of this User.  # noqa: E501


        :return: The invite_status of this User.  # noqa: E501
        :rtype: str
        """
        return self._invite_status

    @invite_status.setter
    def invite_status(self, invite_status):
        """Sets the invite_status of this User.


        :param invite_status: The invite_status of this User.  # noqa: E501
        :type: str
        """
        allowed_values = ["accepted", "invited"]  # noqa: E501
        if invite_status not in allowed_values:
            raise ValueError(
                "Invalid value for `invite_status` ({0}), must be one of {1}".format(  # noqa: E501
                    invite_status, allowed_values
                )
            )

        self._invite_status = invite_status

    @property
    def invite_ticket(self):
        """Gets the invite_ticket of this User.  # noqa: E501


        :return: The invite_ticket of this User.  # noqa: E501
        :rtype: str
        """
        return self._invite_ticket

    @invite_ticket.setter
    def invite_ticket(self, invite_ticket):
        """Sets the invite_ticket of this User.


        :param invite_ticket: The invite_ticket of this User.  # noqa: E501
        :type: str
        """

        self._invite_ticket = invite_ticket

    @property
    def updated_at(self):
        """Gets the updated_at of this User.  # noqa: E501


        :return: The updated_at of this User.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this User.


        :param updated_at: The updated_at of this User.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(User, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, User):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class AwsAccessKeyAccessKey(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "access_key_id": "str",
        "create_date": "datetime",
        "secret_access_key": "str",
        "status": "str",
        "user_name": "str",
    }

    attribute_map = {
        "access_key_id": "AccessKeyId",
        "create_date": "CreateDate",
        "secret_access_key": "SecretAccessKey",
        "status": "Status",
        "user_name": "UserName",
    }

    def __init__(
        self,
        access_key_id=None,
        create_date=None,
        secret_access_key=None,
        status=None,
        user_name=None,
    ):  # noqa: E501
        """AwsAccessKeyAccessKey - a model defined in Swagger"""  # noqa: E501
        self._access_key_id = None
        self._create_date = None
        self._secret_access_key = None
        self._status = None
        self._user_name = None
        self.discriminator = None
        if access_key_id is not None:
            self.access_key_id = access_key_id
        if create_date is not None:
            self.create_date = create_date
        if secret_access_key is not None:
            self.secret_access_key = secret_access_key
        if status is not None:
            self.status = status
        if user_name is not None:
            self.user_name = user_name

    @property
    def access_key_id(self):
        """Gets the access_key_id of this AwsAccessKeyAccessKey.  # noqa: E501


        :return: The access_key_id of this AwsAccessKeyAccessKey.  # noqa: E501
        :rtype: str
        """
        return self._access_key_id

    @access_key_id.setter
    def access_key_id(self, access_key_id):
        """Sets the access_key_id of this AwsAccessKeyAccessKey.


        :param access_key_id: The access_key_id of this AwsAccessKeyAccessKey.  # noqa: E501
        :type: str
        """

        self._access_key_id = access_key_id

    @property
    def create_date(self):
        """Gets the create_date of this AwsAccessKeyAccessKey.  # noqa: E501


        :return: The create_date of this AwsAccessKeyAccessKey.  # noqa: E501
        :rtype: datetime
        """
        return self._create_date

    @create_date.setter
    def create_date(self, create_date):
        """Sets the create_date of this AwsAccessKeyAccessKey.


        :param create_date: The create_date of this AwsAccessKeyAccessKey.  # noqa: E501
        :type: datetime
        """

        self._create_date = create_date

    @property
    def secret_access_key(self):
        """Gets the secret_access_key of this AwsAccessKeyAccessKey.  # noqa: E501


        :return: The secret_access_key of this AwsAccessKeyAccessKey.  # noqa: E501
        :rtype: str
        """
        return self._secret_access_key

    @secret_access_key.setter
    def secret_access_key(self, secret_access_key):
        """Sets the secret_access_key of this AwsAccessKeyAccessKey.


        :param secret_access_key: The secret_access_key of this AwsAccessKeyAccessKey.  # noqa: E501
        :type: str
        """

        self._secret_access_key = secret_access_key

    @property
    def status(self):
        """Gets the status of this AwsAccessKeyAccessKey.  # noqa: E501


        :return: The status of this AwsAccessKeyAccessKey.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this AwsAccessKeyAccessKey.


        :param status: The status of this AwsAccessKeyAccessKey.  # noqa: E501
        :type: str
        """
        allowed_values = ["Active", "Inactive"]  # noqa: E501
        if status not in allowed_values:
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}".format(  # noqa: E501
                    status, allowed_values
                )
            )

        self._status = status

    @property
    def user_name(self):
        """Gets the user_name of this AwsAccessKeyAccessKey.  # noqa: E501


        :return: The user_name of this AwsAccessKeyAccessKey.  # noqa: E501
        :rtype: str
        """
        return self._user_name

    @user_name.setter
    def user_name(self, user_name):
        """Sets the user_name of this AwsAccessKeyAccessKey.


        :param user_name: The user_name of this AwsAccessKeyAccessKey.  # noqa: E501
        :type: str
        """

        self._user_name = user_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(AwsAccessKeyAccessKey, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AwsAccessKeyAccessKey):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
`embedops_authorization_test`
=======================================================================
Unit tests for the authorization retrival and storage for EmbedOps 
* Author(s): Bailey Steinfadt
"""

import os
import pytest
from datetime import timedelta, datetime

from embedops_cli import embedops_authorization
from embedops_cli.eo_types import UnauthorizedUserException, DockerRegistryException


def test_set_and_get_auth_token():
    """testing setting token"""
    test_secret = "SUPER_DUPER_SECRET_TOKEN_SAUCE"
    test_secret_file = ".test_eosecrets.toml"

    embedops_authorization.set_auth_token(test_secret, test_secret_file)
    retrieved_secret = embedops_authorization.get_auth_token(test_secret_file)

    assert test_secret == retrieved_secret

    os.remove(test_secret_file)


def test_reauth_for_docker_token_with_old_date():
    """testing docker exp token old date"""
    old_date = datetime.now() - timedelta(1)
    
    test_secret_file = ".test_eosecrets.toml"

    embedops_authorization.set_docker_expiration_time(old_date, test_secret_file)

    # Get a brand new token/exp date
    # Mimic login to registry (tomorrows date)
    new_exp_date = datetime.now() + timedelta(days=1)

    embedops_authorization.set_docker_expiration_time(new_exp_date, test_secret_file)

    is_valid = embedops_authorization.is_registery_token_valid(test_secret_file)

    assert is_valid == True

    os.remove(test_secret_file)


def test_reauth_for_docker_token_with_no_date(mocker):
    """testing docker exp token no date"""
    test_secret_file = ".test_eosecrets.toml"

    # Get a brand new token/exp date
    # Mimic login to registry (tomorrows date)
    new_exp_date = datetime.now() + timedelta(days=1)
    embedops_authorization.set_docker_expiration_time(new_exp_date, test_secret_file)

    is_valid = embedops_authorization.is_registery_token_valid(test_secret_file)

    assert is_valid == True

    os.remove(test_secret_file)


def test_reauth_for_docker_token_to_fail(mocker):
    """testing docker exp token to fail"""
    test_secret_file = ".test_eosecrets.toml"

    is_valid = embedops_authorization.is_registery_token_valid(test_secret_file)

    assert is_valid == False

    old_date = datetime.now() - timedelta(1)
    test_secret_file = ".test_eosecrets.toml"

    embedops_authorization.set_docker_expiration_time(old_date, test_secret_file)

    is_valid = embedops_authorization.is_registery_token_valid(test_secret_file)

    assert is_valid == False


def test_set_and_get_registry_token():
    """testing setting token"""
    test_access_id = "SOMEID"
    test_secret = "YULE NEFFER GESS WOT"
    test_secret_file = ".test_eosecrets.toml"

    embedops_authorization.set_registry_token(
        test_access_id, test_secret, test_secret_file
    )
    registry_token_data = embedops_authorization.get_registry_token(test_secret_file)

    retrieved_id = registry_token_data["registry_token_id"]
    retrieved_secret = registry_token_data["registry_token_secret"]

    assert test_access_id == retrieved_id
    assert test_secret == retrieved_secret

    os.remove(test_secret_file)


def test_setting_both_tokens():
    """Test that both the registry token and auth token can be read and written together"""

    auth_secret = "SUPER_DUPER_SECRET_TOKN_SAUCE"
    other_auth_secret = "THIS IS THE GOOD ONE"
    registry_id = "SOMEID"
    registry_secret = "YULE NEFFER GESS WOT"

    test_secret_file = ".test_eosecrets.toml"

    embedops_authorization.set_auth_token(auth_secret, test_secret_file)
    embedops_authorization.set_registry_token(
        registry_id, registry_secret, test_secret_file
    )

    read_auth_token = embedops_authorization.get_auth_token(test_secret_file)

    registry_token_data = embedops_authorization.get_registry_token(test_secret_file)

    read_registry_id = registry_token_data["registry_token_id"]
    read_registry_secret = registry_token_data["registry_token_secret"]

    assert auth_secret == read_auth_token
    assert registry_id == read_registry_id
    assert registry_secret == read_registry_secret

    embedops_authorization.set_auth_token(other_auth_secret, test_secret_file)
    read_auth_token = embedops_authorization.get_auth_token(test_secret_file)

    registry_token_data = embedops_authorization.get_registry_token(test_secret_file)

    read_registry_id = registry_token_data["registry_token_id"]
    read_registry_secret = registry_token_data["registry_token_secret"]

    assert other_auth_secret == read_auth_token
    assert registry_id == read_registry_id
    assert registry_secret == read_registry_secret

    os.remove(test_secret_file)


def test_login_to_registry_no_registry_credentials_fails_login(mocker):
    """Test that the function handles non-existent creds correctly"""

    mock_docker_cli_login = mocker.patch(
        "embedops_cli.embedops_authorization.docker_cli_login",
        return_value=2,  # docker cli process returns exit code 2 if login failed
    )

    with pytest.raises(UnauthorizedUserException) as pytest_wrapped_e:
        embedops_authorization.login_to_registry("non-existent-secrets.toml")

    assert pytest_wrapped_e.type == UnauthorizedUserException
    mock_docker_cli_login.assert_not_called()


def test_login_to_registry_incorrect_registry_credentials_fails_login(mocker):
    """Test that the function handles invalid credentials correctly"""

    mocker.patch(
        "embedops_cli.embedops_authorization.get_registry_token",
        # return_value=("BAD_TOKEN_ID_BAD", "BAD_TOKEN_SECRET_BAD")
        return_value=(
            {
                "registry_token_id": "BAD_TOKEN_ID_BAD",
                "registry_token_secret": "BAD_TOKEN_SECRET_BAD",
            }
        ),
    )
    mocker.patch(
        "embedops_cli.embedops_authorization.docker_cli_login",
        return_value=1,  # docker cli process returns exit code 1 if login failed
    )

    with pytest.raises(DockerRegistryException) as pytest_wrapped_e:
        embedops_authorization.login_to_registry()

    assert pytest_wrapped_e.type == DockerRegistryException


def test_login_to_registry(mocker):
    """Test that the function handles a successful login correctly"""
    mock_get_registry_token = mocker.patch(
        "embedops_cli.embedops_authorization.get_registry_token",
        return_value=(
            {
                "registry_token_id": "MAGIC_GOOD_TOKEN_ID",
                "registry_token_secret": "MAGIC_GOOD_TOKEN_SECRET",
            }
        ),
    )
    print(mock_get_registry_token)
    mock_docker_cli_login = mocker.patch(
        "embedops_cli.embedops_authorization.docker_cli_login", return_value=0
    )
    try:
        embedops_authorization.login_to_registry()
    except Exception as exc:
        assert False, f"'test_login_to_registry' raised an exception {exc}"

    mock_get_registry_token.assert_called()
    mock_docker_cli_login.assert_called()



"""A module to handle updating a CI run's sdtatus"""
from logging import getLogger
from sys import stdin
from embedops_cli.config import settings

# import embedops_cli.api as api_lib
from embedops_cli.api import ApiClient, CIRunUpdateProps, Configuration

_logger = getLogger(__name__)


class CiRun:
    """A class for updating the status of a CI run depending on the status of the current CI job"""

    @staticmethod
    def _update_ci_run(status_str):

        try:
            configuration = Configuration()
            configuration.api_key["X-API-Key"] = settings.api_repo_key
            configuration.host = f"{settings.host}/api/v1"

            # create an instance of the API class
            api_instance = ApiClient(configuration)
            ci_run = CIRunUpdateProps()
            ci_run.status = status_str
            ci_run.pipeline_url = settings.job_url
            updated_run = api_instance.update_ci_run_from_ci(ci_run, settings.run_id)

            _logger.debug("Updated CI run:")
            _logger.debug(updated_run)

            del api_instance  # kills the thread pool
        except AttributeError as exc:
            _logger.info("No API Repo Key provided. CI run will not be updated.")
            _logger.info(exc)

    @staticmethod
    def _previous_return():
        return int(stdin.readline())

    def finalize_ci_run(self):
        """Update the status of the current CI run based on the value of a standard in read"""
        previous_return = self._previous_return()
        if previous_return != 0:
            self._update_ci_run("failure")
        else:
            self._update_ci_run("success")


def main():
    """Entrypoint for the `eotools-update-run` tool"""
    ci_run = CiRun()
    ci_run.finalize_ci_run()



"""
`api_interface_test`
=======================================================================
Unit tests for code to upload XML files to the API
* Author(s): Bryan Siepert
"""

import uuid
from unittest.mock import Mock
from embedops_cli.config import settings
from embedops_cli.api import CIRunUpdateProps
from embedops_cli.eotools.update_run import main as update_run_main
from embedops_cli.eotools.create_run import CIRun
import pytest


class FakeResponse:
    """Fake function return"""

    @classmethod
    def ret_zero(cls):
        """Just returns zero"""
        return "0"


class FakeConfig:
    """Fake config class"""

    def __init__(self):
        self._run_id = uuid.uuid1()

    @property
    def run_id(self):
        """Run id for fake config class"""
        return self._run_id


class EOToolsTests:
    @pytest.fixture(autouse=True)
    def configure_env(self, monkeypatch, mocker):
        monkeypatch.setenv("CI", "true")
        monkeypatch.setenv("EMBEDOPS_API_REPO_KEY", "special_pwefix")
        monkeypatch.setenv("EMBEDOPS_ANALYSIS_TYPE", "memusage")
        monkeypatch.setenv("BITBUCKET_COMMIT", "abcd1234")
        monkeypatch.setenv("BITBUCKET_BRANCH", "main")
        self.ci_run_caller = mocker.patch(
            "embedops_cli.eotools.create_run.CIRun.create_run"
        )
        self.logger_mock = mocker.patch("embedops_cli.eotools.create_run._logger")
        self.ci_run = CIRun()
        yield
        del self.ci_run

    def test_successful_job_sets_ci_url(self, mocker):

        mocker.patch(
            "embedops_cli.eotools.update_run.CiRun._previous_return", return_value=0
        )

        # create the mock to check how update_ci_run_from_ci is called
        api_client_mock = Mock()
        api_client_mock.update_ci_run_from_ci.return_value = object()

        mocker.patch(
            "embedops_cli.eotools.update_run.ApiClient", return_value=api_client_mock
        )

        update_run_main()

        calls = api_client_mock.update_ci_run_from_ci.call_args_list
        assert len(calls) == 1

        call_args = calls[0][0]
        ci_properties = call_args[0]
        ci_run_id = call_args[1]

        assert isinstance(ci_properties, CIRunUpdateProps)
        assert ci_run_id == settings.run_id
        assert ci_properties.pipeline_url == settings.job_url

    def test_api_client_configured(self, mocker):

        self.config_dict = mocker.patch.dict(
            self.config_class.api_key, {"X-API-Key": None}
        )
        api_client = mocker.patch("embedops_cli.api.get_client")
        with pytest.raises(SystemExit) as pytest_wrapped_e:
            create_entry()
        assert pytest_wrapped_e.type == SystemExit
        assert pytest_wrapped_e.value.code == 0

        # This tests that the configuration object's 'api_key' attribute dict's
        # 'X-API-Key' key is set to the specified API key
        self.config_class().api_key.__setitem__.assert_called_with(
            "X-API-Key", "special_pwefix"
        )
        assert self.config_class().host == self.expected_host + "/api/v1"

        args, _kwargs = api_client().create_ci_run_from_ci.call_args
        first_call = args[0]
        assert first_call.type == self.expected_type
        assert first_call.commit_id == self.expected_commit_id
        assert first_call.branch == self.expected_branch



"""
`embedops_cli_test`
=======================================================================
Unit tests for the CLI interface for EmbedOps tools
* Author(s): Bailey Steinfadt
"""

import os
import pytest
from click.testing import CliRunner
from embedops_cli import embedops_cli
from tests import BBYML_FILENAME, GLYML_FILENAME
from embedops_cli.eo_types import UnauthorizedUserException

NOT_A_FILENAME = "not_a_file.yaml"
GHYML_TEST_DETECTION_FILENAME = ".github/workflows/test-detection-.github-ci.yml"


@pytest.fixture
def change_test_dir(request):
    """
    A function-level fixture that changes to the test case directory,
    run the test (yield), then change back to the calling directory to
    avoid side-effects.
    """
    os.chdir(request.fspath.dirname)
    yield
    os.chdir(request.config.invocation_dir)


def test_version_command():
    """Learned how to write tests for click by testing build in version command"""
    runner = CliRunner()
    result = runner.invoke(embedops_cli.embedops_cli, ["--version"])
    assert result.exit_code == 0
    assert result.output[:21] == "embedops-cli, version"


def test_help_command():
    """Learned how to write tests for click by testing built in help command"""
    runner = CliRunner()
    h_result = runner.invoke(embedops_cli.embedops_cli, "-h")
    assert h_result.exit_code == 0
    help_result = runner.invoke(embedops_cli.embedops_cli, "--help")
    assert help_result.exit_code == 0
    assert help_result.output == h_result.output
    halp_result = runner.invoke(embedops_cli.embedops_cli, "--halp")
    assert halp_result.exit_code == 0
    assert halp_result.output == h_result.output


def test_jobs_command():
    """Test the top level jobs group"""
    runner = CliRunner()
    result = runner.invoke(embedops_cli.embedops_cli, "jobs")
    assert result.exit_code == 0
    result = runner.invoke(
        embedops_cli.embedops_cli, ["jobs", "--filename", "not_a_file.yaml"]
    )
    assert result.exit_code == 2


def test_show_jobs_no_filename():
    """Test the show job list command no filename"""
    runner = CliRunner()
    result = runner.invoke(embedops_cli.embedops_cli, ["jobs", "show"])

    assert result.exit_code == 0


def test_show_jobs_nonexistent_filename():
    """Test the show job list command with a nonexistent file"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli, ["jobs", "--filename", "not_a_file.yaml", "show"]
    )
    assert result.exit_code == 2


def test_show_jobs_not_yaml_filename():
    """Test the show job list command with a non-yaml filename"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli, ["jobs", "--filename", "tests/README.md", "show"]
    )
    assert result.exit_code == 1


def test_show_jobs():
    """Test the show job list command correct syntax"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli,
        ["jobs", "--filename", BBYML_FILENAME, "show"],
    )
    assert result.exit_code == 0

    result = runner.invoke(
        embedops_cli.embedops_cli,
        ["jobs", "--filename", GLYML_FILENAME, "show"],
    )
    assert result.exit_code == 0


def test_run_jobs_no_job_name_or_filename():
    """Test the run job command with no job or filename"""

    runner = CliRunner()
    result = runner.invoke(embedops_cli.embedops_cli, ["jobs", "run"])
    assert result.exit_code == 2


def test_run_jobs_no_job_name():
    """Test the run job list command with no job name"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli,
        ["jobs", "--filename", BBYML_FILENAME, "run"],
    )
    assert result.exit_code == 2


def test_run_jobs_nonexistent_file():
    """Test the run job command with a nonexistent file"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli,
        ["jobs", "--filename", NOT_A_FILENAME, "run", "build"],
    )
    assert result.exit_code == 2


def test_run_jobs_not_yaml_filename():
    """Test the run job command with a non-yaml filename"""
    runner = CliRunner()
    result = runner.invoke(
        embedops_cli.embedops_cli,
        ["jobs", "--filename", "tests/README.md", "run", "build"],
    )
    assert result.exit_code == 1


def test_stale_token_prompts_login(mocker):
    """Test that a no-longer-valid auth token prompts"""
    token_status = mocker.patch(
        "embedops_cli.embedops_cli.embedops_authorization.check_token"
    )
    token_status.return_value = False

    do_login = mocker.patch(
        "embedops_cli.embedops_cli.embedops_authorization.request_authorization"
    )
    do_login.side_effect = UnauthorizedUserException()
    runner = CliRunner()
    result = runner.invoke(embedops_cli.embedops_cli, "login")
    do_login.assert_called()

    assert result.exit_code != 0

    # mock `check user` to return false
    # verify that "do login" is called
    # verify that if "do login" is successful that the return value is 0



#!/usr/bin/env python
"""
`ci_run.py`
=======================================================================
A small script to create and update EmbedOps CI run records
* Author(s): Bryan Siepert
"""
import sys
import importlib
import logging
from os import getenv
from embedops_cli.api import (
    Configuration,
    CIRunCreateProps,
    CIRunUpdateProps,
    rest,
)
from embedops_cli.api import get_client
import embedops_cli.config

_logger = logging.getLogger("create_run")


class CIRun:
    """A wrapper for creating and updating CIRun records"""

    def __init__(self):
        # pylint: disable=import-outside-toplevel

        importlib.reload(embedops_cli.config)
        self._settings = embedops_cli.config.settings

    # create().id
    def create_main(self):
        """The main entrypoint for the module, to allow for binary-izing"""

        env_error = False

        if getenv("CI") is None or getenv("CI") != "true":
            _logger.info("LOCAL")
            sys.exit(0)

        if self._settings.get("commit") is None:
            _logger.error("ERROR: No commit id envvar found")
            env_error = True

        if self._settings.get("branch") is None:
            _logger.error("ERROR: No branch name envvar found")
            env_error = True

        if self._settings.get("analysis_type") is None:
            _logger.error("ERROR: No analysis type envvar found")
            env_error = True

        if self._settings.get("api_repo_key") is None:
            _logger.error("ERROR: No EMBEDOPS_API_REPO_KEY envvar found")
            env_error = True

        if env_error is True:
            sys.exit(1)

        analysis_type = self._settings.analysis_type
        commit_id = self._settings.commit
        branch_name = self._settings.branch
        pipeline_id = self._settings.get("pipeline_id")
        job_name = self._settings.get("job_name")

        return self.create(analysis_type, commit_id, branch_name, pipeline_id, job_name)

    @staticmethod
    def create(analysis_type, commit_id, branch_name, pipeline_id=None, job_name=None):

        """Use the branch name, commit sha, analysis type, job_name,
        and pipeline id to create a new ciRun instance"""

        api_client = get_client()
        create_properties = CIRunCreateProps(
            branch=branch_name,
            commit_id=commit_id,
            type=analysis_type,
            pipeline_id=pipeline_id,
            name=job_name,
        )

        response = None

        try:
            response = api_client.create_ci_run_from_ci(create_properties)
        except (ValueError, TypeError, rest.ApiException) as err:
            _logger.error(err.args[0])
            sys.exit(1)
        finally:
            # this kills the thread pool so the process can exit
            del api_client
        return response

    def _update_ci_run(self, status_str):

        try:
            api_client = get_client()
            configuration = Configuration()
            configuration.api_key["X-API-Key"] = self._settings.api_repo_key
            configuration.host = f"{self._settings.host}/api/v1"

            ci_run = CIRunUpdateProps()
            ci_run.status = status_str
            ci_run.pipeline_url = self._settings.job_url
            updated_run = api_client.update_ci_run_from_ci(
                ci_run, self._settings.run_id
            )

            _logger.debug("Updated CI run:")
            _logger.debug(updated_run)

            del api_client  # kills the thread pool
        except AttributeError as exc:
            _logger.info("No API Repo Key provided. CI run will not be updated.")
            _logger.info(exc)

    @staticmethod
    def _previous_return():
        return int(sys.argv[1])

    def finalize_ci_run(self):
        """Update the status of the current CI run based on the value of a standard in read"""
        previous_return = self._previous_return()
        if previous_return != 0:
            self._update_ci_run("failure")
        else:
            self._update_ci_run("success")


def update_entry():
    """Entrypoint for the `eotools-update-run` tool"""
    ci_run = CIRun()
    ci_run.finalize_ci_run()


def create_entry():
    """Entrypoint for the `eotools-create-run` tool"""
    ci_run = CIRun()
    new_ci_run = ci_run.create_main()
    _logger.info(new_ci_run.id)
    # DO NOT REMOVE. need to output ID for eotools-create-run
    print(new_ci_run.id)
    sys.exit(0)


if __name__ == "__main__":
    create_entry()



"""
`gl_parser`
=======================================================================
Parser to pull job contexts from .gitlab-ci.yml files
* Author(s): Bailey Steinfadt
"""
import logging
import re
from embedops_cli.yaml_tools import open_yaml
from ..eo_types import LocalRunContext, BadYamlFileException

_logger = logging.getLogger(__name__)

_GITLAB_RESERVED_KEYWORDS = [
    "stages",
    "image",
    "workflow",
    "include",
    "services",
    "types",
    "before_script",
    "after_script",
    "variables",
    "cache",
    "default",
    "artifacts",
    "interruptable",
    "retry",
    "tags",
    "timeout",
]


def get_job_name_list(glyml_filename: str) -> list:
    """Get list of job names from the given YAML object"""

    try:
        glyml = open_yaml(glyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_name_list = []

    try:
        for item in glyml.items():
            # reserved GitLab yaml tags
            if item[0] not in _GITLAB_RESERVED_KEYWORDS:
                job_name_list.append(item[0])

        if not all(isinstance(job_name, str) for job_name in job_name_list):
            raise BadYamlFileException()

        return job_name_list
    except (KeyError) as err:
        raise BadYamlFileException() from err


def get_job_list(glyml_filename: str) -> list:
    """Return the list of LocalRunContexts found in the given yaml object"""

    try:
        glyml = open_yaml(glyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_list = []

    try:
        default_image = ""
        default_var_dict = {}
        for tag, value in glyml.items():
            # default image can be at top level image:,
            # under image:name:, or default:image:
            if tag == "image":
                if "name" in value:
                    default_image = value["name"]
                else:
                    default_image = value
            if tag == "default":
                if "image" in value:
                    default_image = value["image"]
                # update default_var_dict (in case top level vars came first)
                if "variables" in value:
                    default_var_dict.update(value["variables"])

            # update default_var_dict (in case default:variables: came first)
            if tag == "variables":
                default_var_dict.update(value)

            # reserved GitLab yaml tags are the only things that aren't jobs
            if tag not in _GITLAB_RESERVED_KEYWORDS:

                job_ctx = _parse_job_context(
                    tag, value, default_image, default_var_dict
                )

                job_list.append(job_ctx)
        return job_list
    except (TypeError, KeyError, BadYamlFileException) as err:
        raise BadYamlFileException() from err


def _parse_job_context(yaml_tag, yaml_value, default_image, default_var_dict):
    # Image defined in job overrides any default image
    image = ""

    if "image" in yaml_value:
        if "name" in yaml_value["image"]:
            image = yaml_value["image"]["name"]
        else:

            image = yaml_value["image"]
    else:
        image = default_image

    # Does not need to have a script, can be empty
    # TODO: parse before and after scripts
    # TODO: parse the | token appropriately
    script_list = []
    if "script" in yaml_value:
        if isinstance(yaml_value["script"], list):
            for line in yaml_value["script"]:
                script_list.append(line)
        elif isinstance(yaml_value["script"], str):
            script_list = yaml_value["script"].split("\n")
            if script_list[-1] == "":
                # Remove the last empty command
                script_list.pop(-1)
        else:
            raise BadYamlFileException()

    # Does not necessarily have variables
    # if it does, add them (via update()) to the default_var_dict
    var_dict = {}
    var_dict.update(default_var_dict)
    if "variables" in yaml_value:
        dereferenced_job_vars = _dereference_vars(
            yaml_value["variables"], default_var_dict
        )
        var_dict.update(dereferenced_job_vars)
    return LocalRunContext(yaml_tag, image, script_list, var_dict)


def _dereference_vars(raw_vars: dict, global_vars: dict) -> dict:
    dereferenced_vars = {}
    for raw_key in raw_vars.keys():
        dereferenced_value = raw_vars[raw_key]
        for global_key in global_vars.keys():
            just_dollar = rf"\${global_key}"
            dollar_and_braces = rf"\${{{global_key}}}"
            if re.search(just_dollar, raw_vars[raw_key]) is not None:
                dereferenced_value = re.sub(
                    just_dollar, global_vars[global_key], raw_vars[raw_key]
                )
            if re.search(dollar_and_braces, raw_vars[raw_key]) is not None:
                dereferenced_value = re.sub(
                    dollar_and_braces, global_vars[global_key], raw_vars[raw_key]
                )

        dereferenced_vars[raw_key] = dereferenced_value
    return dereferenced_vars



"""
`bb_parser`
=======================================================================
Parser to pull job contexts from bitbucket-pipelines.yml files
* Author(s): Bailey Steinfadt
"""
import logging
from embedops_cli.yaml_tools import open_yaml
from ..eo_types import BadYamlFileException, LocalRunContext

_logger = logging.getLogger(__name__)


def get_job_name_list(bbyml_filename: str):
    """Get list of job names from the given YAML object"""

    try:
        bbyml = open_yaml(bbyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_name_list = []

    try:
        for job_def in bbyml["pipelines"]["default"]:
            if "step" in job_def:
                job_name_list.append(job_def["step"]["name"])

            elif "parallel" in job_def:
                for par_step in job_def["parallel"]:
                    job_name_list.append(par_step["step"]["name"])

        if not all(isinstance(job_name, str) for job_name in job_name_list):
            raise BadYamlFileException()

        return job_name_list
    except (KeyError) as err:
        raise BadYamlFileException() from err


def get_job_list(bbyml_filename: str) -> list:
    """Return the list of LocalRunContexts found in the given yaml object"""

    try:
        bbyml = open_yaml(bbyml_filename)
    except BadYamlFileException as exc:
        raise BadYamlFileException() from exc

    job_list = []

    try:
        default_image = "atlassian/default-image:latest"
        if "image" in bbyml:
            default_image = bbyml["image"]

        job_list = _parse_job_context(bbyml, default_image)

        return job_list
    except (KeyError, AttributeError, TypeError) as err:
        raise BadYamlFileException() from err


def _parse_job_context(bbyml, default_image):
    job_list = []

    # TODO: parse for other pipelines (branches, tags, pull-requests, custom)
    # TODO: parse for definitions and YAML anchors
    for job_def in bbyml["pipelines"]["default"]:
        if "step" in job_def:
            script_list = []

            if "script" in job_def["step"]:
                for line in job_def["step"]["script"]:
                    script_list.append(line)
            image = ""
            if "image" in job_def["step"]:
                image = job_def["step"]["image"]["name"]
            else:
                image = default_image
            job_list.append(
                LocalRunContext(job_def["step"]["name"], image, script_list)
            )

        elif "parallel" in job_def:
            for par_step in job_def["parallel"]:
                script_list = []

                if "script" in par_step["step"]:
                    for line in par_step["step"]["script"]:
                        script_list.append(line)
                image = ""
                if "image" in par_step["step"]:
                    image = par_step["step"]["image"]["name"]
                job_list.append(
                    LocalRunContext(par_step["step"]["name"], image, script_list)
                )

    return job_list



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class RepoCreateProps(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "name": "str",
        "uri": "str",
        "ci_provider": "str",
        "ci_o_auth_token_id": "str",
        "desc": "str",
        "default_branch": "str",
    }

    attribute_map = {
        "name": "name",
        "uri": "uri",
        "ci_provider": "ciProvider",
        "ci_o_auth_token_id": "ciOAuthTokenId",
        "desc": "desc",
        "default_branch": "defaultBranch",
    }

    def __init__(
        self,
        name=None,
        uri=None,
        ci_provider=None,
        ci_o_auth_token_id=None,
        desc=None,
        default_branch=None,
    ):  # noqa: E501
        """RepoCreateProps - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._uri = None
        self._ci_provider = None
        self._ci_o_auth_token_id = None
        self._desc = None
        self._default_branch = None
        self.discriminator = None
        self.name = name
        self.uri = uri
        if ci_provider is not None:
            self.ci_provider = ci_provider
        if ci_o_auth_token_id is not None:
            self.ci_o_auth_token_id = ci_o_auth_token_id
        if desc is not None:
            self.desc = desc
        self.default_branch = default_branch

    @property
    def name(self):
        """Gets the name of this RepoCreateProps.  # noqa: E501


        :return: The name of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this RepoCreateProps.


        :param name: The name of this RepoCreateProps.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError(
                "Invalid value for `name`, must not be `None`"
            )  # noqa: E501

        self._name = name

    @property
    def uri(self):
        """Gets the uri of this RepoCreateProps.  # noqa: E501


        :return: The uri of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._uri

    @uri.setter
    def uri(self, uri):
        """Sets the uri of this RepoCreateProps.


        :param uri: The uri of this RepoCreateProps.  # noqa: E501
        :type: str
        """
        if uri is None:
            raise ValueError(
                "Invalid value for `uri`, must not be `None`"
            )  # noqa: E501

        self._uri = uri

    @property
    def ci_provider(self):
        """Gets the ci_provider of this RepoCreateProps.  # noqa: E501


        :return: The ci_provider of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._ci_provider

    @ci_provider.setter
    def ci_provider(self, ci_provider):
        """Sets the ci_provider of this RepoCreateProps.


        :param ci_provider: The ci_provider of this RepoCreateProps.  # noqa: E501
        :type: str
        """
        allowed_values = ["github", "gitlab", "bitbucket", ""]  # noqa: E501
        if ci_provider not in allowed_values:
            raise ValueError(
                "Invalid value for `ci_provider` ({0}), must be one of {1}".format(  # noqa: E501
                    ci_provider, allowed_values
                )
            )

        self._ci_provider = ci_provider

    @property
    def ci_o_auth_token_id(self):
        """Gets the ci_o_auth_token_id of this RepoCreateProps.  # noqa: E501


        :return: The ci_o_auth_token_id of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._ci_o_auth_token_id

    @ci_o_auth_token_id.setter
    def ci_o_auth_token_id(self, ci_o_auth_token_id):
        """Sets the ci_o_auth_token_id of this RepoCreateProps.


        :param ci_o_auth_token_id: The ci_o_auth_token_id of this RepoCreateProps.  # noqa: E501
        :type: str
        """

        self._ci_o_auth_token_id = ci_o_auth_token_id

    @property
    def desc(self):
        """Gets the desc of this RepoCreateProps.  # noqa: E501


        :return: The desc of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._desc

    @desc.setter
    def desc(self, desc):
        """Sets the desc of this RepoCreateProps.


        :param desc: The desc of this RepoCreateProps.  # noqa: E501
        :type: str
        """

        self._desc = desc

    @property
    def default_branch(self):
        """Gets the default_branch of this RepoCreateProps.  # noqa: E501


        :return: The default_branch of this RepoCreateProps.  # noqa: E501
        :rtype: str
        """
        return self._default_branch

    @default_branch.setter
    def default_branch(self, default_branch):
        """Sets the default_branch of this RepoCreateProps.


        :param default_branch: The default_branch of this RepoCreateProps.  # noqa: E501
        :type: str
        """
        if default_branch is None:
            raise ValueError(
                "Invalid value for `default_branch`, must not be `None`"
            )  # noqa: E501

        self._default_branch = default_branch

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(RepoCreateProps, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RepoCreateProps):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""
`telemetry`
=======================================================================
Utility for telemetry
* Author(s): Jimmy Gomez
"""
import requests
from embedops_cli.config import settings
from .embedops_authorization import get_auth_token

measurement_id = settings.get("measurement_id")
api_secret = settings.get("api_secret")
CLIENT_ID = "embedops-cli"


def login_event():
    """Login Event"""
    event = {"name": "login"}
    send_telemetry(event)


def command_event(command, subcommands):
    """Format CLI command event for telemetry and send"""
    event = {"name": command, "params": subcommands}
    send_telemetry(event)


def send_telemetry(event):
    """Sends telemetry to Embedops telemetry endpoint"""
    token = get_auth_token()
    headers = {"Authorization": f"Bearer {token}"}
    host = settings.get("host")
    url = f"{host}/api/v1/users/myself/cli-telemetry"
    requests.post(url, headers=headers, json=event)



# coding: utf-8

"""
    EmbedOps API

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: 1.0.0
    Contact: support@embedops.com
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class GitlabAccessTokenToken(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        "access_level": "float",
        "active": "bool",
        "created_at": "str",
        "expires_at": "str",
        "id": "float",
        "name": "str",
        "revoked": "bool",
        "scopes": "list[str]",
        "token": "str",
        "user_id": "float",
    }

    attribute_map = {
        "access_level": "access_level",
        "active": "active",
        "created_at": "created_at",
        "expires_at": "expires_at",
        "id": "id",
        "name": "name",
        "revoked": "revoked",
        "scopes": "scopes",
        "token": "token",
        "user_id": "user_id",
    }

    def __init__(
        self,
        access_level=None,
        active=None,
        created_at=None,
        expires_at=None,
        id=None,
        name=None,
        revoked=None,
        scopes=None,
        token=None,
        user_id=None,
    ):  # noqa: E501
        """GitlabAccessTokenToken - a model defined in Swagger"""  # noqa: E501
        self._access_level = None
        self._active = None
        self._created_at = None
        self._expires_at = None
        self._id = None
        self._name = None
        self._revoked = None
        self._scopes = None
        self._token = None
        self._user_id = None
        self.discriminator = None
        if access_level is not None:
            self.access_level = access_level
        if active is not None:
            self.active = active
        if created_at is not None:
            self.created_at = created_at
        if expires_at is not None:
            self.expires_at = expires_at
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if revoked is not None:
            self.revoked = revoked
        if scopes is not None:
            self.scopes = scopes
        if token is not None:
            self.token = token
        if user_id is not None:
            self.user_id = user_id

    @property
    def access_level(self):
        """Gets the access_level of this GitlabAccessTokenToken.  # noqa: E501


        :return: The access_level of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: float
        """
        return self._access_level

    @access_level.setter
    def access_level(self, access_level):
        """Sets the access_level of this GitlabAccessTokenToken.


        :param access_level: The access_level of this GitlabAccessTokenToken.  # noqa: E501
        :type: float
        """
        allowed_values = [10, 20]  # noqa: E501
        if access_level not in allowed_values:
            raise ValueError(
                "Invalid value for `access_level` ({0}), must be one of {1}".format(  # noqa: E501
                    access_level, allowed_values
                )
            )

        self._access_level = access_level

    @property
    def active(self):
        """Gets the active of this GitlabAccessTokenToken.  # noqa: E501


        :return: The active of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: bool
        """
        return self._active

    @active.setter
    def active(self, active):
        """Sets the active of this GitlabAccessTokenToken.


        :param active: The active of this GitlabAccessTokenToken.  # noqa: E501
        :type: bool
        """

        self._active = active

    @property
    def created_at(self):
        """Gets the created_at of this GitlabAccessTokenToken.  # noqa: E501


        :return: The created_at of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: str
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this GitlabAccessTokenToken.


        :param created_at: The created_at of this GitlabAccessTokenToken.  # noqa: E501
        :type: str
        """

        self._created_at = created_at

    @property
    def expires_at(self):
        """Gets the expires_at of this GitlabAccessTokenToken.  # noqa: E501


        :return: The expires_at of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: str
        """
        return self._expires_at

    @expires_at.setter
    def expires_at(self, expires_at):
        """Sets the expires_at of this GitlabAccessTokenToken.


        :param expires_at: The expires_at of this GitlabAccessTokenToken.  # noqa: E501
        :type: str
        """

        self._expires_at = expires_at

    @property
    def id(self):
        """Gets the id of this GitlabAccessTokenToken.  # noqa: E501


        :return: The id of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: float
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this GitlabAccessTokenToken.


        :param id: The id of this GitlabAccessTokenToken.  # noqa: E501
        :type: float
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this GitlabAccessTokenToken.  # noqa: E501


        :return: The name of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this GitlabAccessTokenToken.


        :param name: The name of this GitlabAccessTokenToken.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def revoked(self):
        """Gets the revoked of this GitlabAccessTokenToken.  # noqa: E501


        :return: The revoked of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: bool
        """
        return self._revoked

    @revoked.setter
    def revoked(self, revoked):
        """Sets the revoked of this GitlabAccessTokenToken.


        :param revoked: The revoked of this GitlabAccessTokenToken.  # noqa: E501
        :type: bool
        """

        self._revoked = revoked

    @property
    def scopes(self):
        """Gets the scopes of this GitlabAccessTokenToken.  # noqa: E501


        :return: The scopes of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: list[str]
        """
        return self._scopes

    @scopes.setter
    def scopes(self, scopes):
        """Sets the scopes of this GitlabAccessTokenToken.


        :param scopes: The scopes of this GitlabAccessTokenToken.  # noqa: E501
        :type: list[str]
        """
        allowed_values = [
            "api",
            "read_api",
            "read_registry",
            "write_registry",
            "read_repository",
            "write_repository",
        ]  # noqa: E501
        if not set(scopes).issubset(set(allowed_values)):
            raise ValueError(
                "Invalid values for `scopes` [{0}], must be a subset of [{1}]".format(  # noqa: E501
                    ", ".join(
                        map(str, set(scopes) - set(allowed_values))
                    ),  # noqa: E501
                    ", ".join(map(str, allowed_values)),
                )
            )

        self._scopes = scopes

    @property
    def token(self):
        """Gets the token of this GitlabAccessTokenToken.  # noqa: E501


        :return: The token of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: str
        """
        return self._token

    @token.setter
    def token(self, token):
        """Sets the token of this GitlabAccessTokenToken.


        :param token: The token of this GitlabAccessTokenToken.  # noqa: E501
        :type: str
        """

        self._token = token

    @property
    def user_id(self):
        """Gets the user_id of this GitlabAccessTokenToken.  # noqa: E501


        :return: The user_id of this GitlabAccessTokenToken.  # noqa: E501
        :rtype: float
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this GitlabAccessTokenToken.


        :param user_id: The user_id of this GitlabAccessTokenToken.  # noqa: E501
        :type: float
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(
                    map(lambda x: x.to_dict() if hasattr(x, "to_dict") else x, value)
                )
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(
                    map(
                        lambda item: (item[0], item[1].to_dict())
                        if hasattr(item[1], "to_dict")
                        else item,
                        value.items(),
                    )
                )
            else:
                result[attr] = value
        if issubclass(GitlabAccessTokenToken, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GitlabAccessTokenToken):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""This module provides a central store for configuration data, drawing from environment variables
configuration files, and other sources as needed"""
import os
from dynaconf import Dynaconf

SETTINGS_FILES = ["settings.toml", ".secrets.toml"]

# get an absolute path to the director that this file is in
abs_current_dir = os.path.dirname(os.path.abspath(__file__))

# prepend the settings filenames with an absolute path to the current directory
# by providing an absolute path, dynaconf will load it directly instead of
# searching for it
settings_paths = [
    os.path.join(abs_current_dir, settings_file) for settings_file in SETTINGS_FILES
]

settings = Dynaconf(
    load_dotenv=True,
    envvar_prefix="EMBEDOPS",
    environments=True,
    settings_files=settings_paths,
)

# for some reason, we have to set the loaders separately to be able to use `.configure` to
# change them for tests
settings.configure(
    LOADERS_FOR_DYNACONF=[
        "dynaconf.loaders.yaml_loader",
        "dynaconf.loaders.env_loader",
        "embedops_cli.ci_config_loader",
    ]
)



"""Provides a custom loader to load the appropriate CI provider specific environment variables"""
from os import environ

ci_loader_conf = {
    "bitbucket": {
        # REF https://support.atlassian.com/bitbucket-cloud/docs/variables-and-secrets/
        "commit": "BITBUCKET_COMMIT",
        "branch": "BITBUCKET_BRANCH",
        "pipeline_id": "BITBUCKET_PIPELINE_UUID",
        "job_url": {
            "format": "{0:s}/addon/pipelines/home#!/results/{1:s}/steps/{2:s}",
            "variables": [
                "BITBUCKET_GIT_HTTP_ORIGIN",
                "BITBUCKET_BUILD_NUMBER",
                "BITBUCKET_STEP_UUID",
            ],
        },
    },
    "github": {
        # REF https://docs.github.com/en/actions/learn-github-actions/environment-variables
        "commit": "GITHUB_SHA",
        "branch": "GITHUB_REF_NAME",
        "pipeline_id": "GITHUB_RUN_ID",
        "job_url": {
            "format": "{0:s}/{1:s}/actions/runs/{2!s:s}",
            "variables": [
                "GITHUB_SERVER_URL",
                "GITHUB_REPOSITORY",
                "GITHUB_RUN_ID",
            ],
        },
    },
    "gitlab": {
        # REF https://docs.gitlab.com/ee/ci/variables/predefined_variables.html
        "commit": "CI_COMMIT_SHA",
        "branch": "CI_COMMIT_REF_NAME",
        "pipeline_id": "CI_PIPELINE_ID",
        "job_url": {
            "format": "{0:s}",
            "variables": [
                "CI_JOB_URL",
            ],
        },
    },
}
# this is a required function signature so there are going to be unused arguments
# pylint: disable=unused-argument
def load(obj, env=None, silent=True, key=None, filename=None):
    """
    Reads and loads in to "obj" a single key or all keys from source
    :param obj: the settings instance
    :param env: settings current env (upper case) default='DEVELOPMENT'
    :param silent: if errors should raise
    :param key: if defined load a single key, else load all from `env`
    :param filename: Custom filename to load (useful for tests)
    :return: None
    """
    # Load data from your custom data source (file, database, memory etc)
    # use `obj.set(key, value)` or `obj.update(dict)` to load data
    # use `obj.find_file('filename.ext')` to find the file in search tree
    # Return nothing

    ci_config_dict = {"provider": "LOCAL"}
    for ci_provider, ci_provider_env_cfg in ci_loader_conf.items():
        if not ci_provider_env_cfg["commit"] in environ:
            continue

        ci_config_dict["provider"] = ci_provider
        ci_config_dict["commit"] = environ.get(ci_provider_env_cfg["commit"])
        ci_config_dict["branch"] = environ.get(ci_provider_env_cfg["branch"])
        ci_config_dict["pipeline_id"] = environ.get(ci_provider_env_cfg["pipeline_id"])

        job_url_cnf = ci_provider_env_cfg["job_url"]
        ci_config_dict["job_url"] = job_url_cnf["format"].format(
            *[environ[env_var] for env_var in job_url_cnf["variables"]]
        )
        break
    obj.update(ci_config_dict)



"""
`embedops_authorization`
=======================================================================
Managing the user login functions
* Author(s): Bailey Steinfadt
"""
import os
import base64
import pathlib
import webbrowser
from time import sleep
import logging
import subprocess
from datetime import datetime
from operator import itemgetter
from urllib.parse import urlparse
import requests
import dotenv
import click
from dynaconf import Dynaconf
from dynaconf.loaders import toml_loader
import boto3
from embedops_cli.api.configuration import Configuration
from embedops_cli.api.rest import ApiException
from embedops_cli.api.api_client import ApiClient
from embedops_cli.eo_types import (
    DockerRegistryException,
    DockerNotRunningException,
    LoginFailureException,
    UnauthorizedUserException,
    UnknownDockerException,
)
from embedops_cli.api.default_api import DefaultApi
from embedops_cli.config import settings

_logger = logging.getLogger(__name__)

EMBEDOPS_REGISTRY = "https://registry.embedops.com"
user_secrets = os.path.join(os.path.expanduser("~"), ".eosecrets.toml")


def get_auth_token(secrets_file=user_secrets) -> str:
    """Retrieve the Auth0 token from user secrets"""
    key = None
    if not os.path.exists(secrets_file):
        return key
    secrets_file_settings = Dynaconf(SETTINGS_FILES=[secrets_file])
    try:
        key = secrets_file_settings.EMBEDOPS_AUTH_TOKEN
    except AttributeError:
        # eat the attribute error because it's expected when the user has not logged in
        pass
    return key


def set_auth_token(auth_token: str, secrets_file=user_secrets):
    """Set the Auth0 token in user secrets"""
    try:
        toml_loader.write(secrets_file, {"EMBEDOPS_AUTH_TOKEN": auth_token}, merge=True)
    except (IOError) as exc:
        raise LoginFailureException from exc


def set_docker_expiration_time(expires_at, secrets_file=user_secrets):
    """Set the Auth0 token in user secrets"""
    try:
        toml_loader.write(
            secrets_file,
            {"EMBEDOPS_ECR_TOKEN_EXPIRES_AT": expires_at.timestamp()},
            merge=True,
        )
    except (IOError) as exc:
        raise LoginFailureException from exc


def fetch_registry_token() -> str:
    """Retrieve a GitLab token for the user's group and store it"""
    user_client = get_user_client()

    user = user_client.get_my_user()
    if len(user.groups) < 1:
        raise UnauthorizedUserException

    first_group_membership = user.groups[0]
    org_id = first_group_membership.group.org_id

    token_record = user_client.get_aws_access_key_for_org(org_id)
    set_registry_token(
        token_record.access_key.access_key_id, token_record.access_key.secret_access_key
    )


def get_docker_expiration_time(secrets_file=user_secrets):
    """Retrieve docker exp date"""
    registry_token_expires_at = None

    # Return false since variable is not set
    if not os.path.exists(secrets_file):
        return registry_token_expires_at

    secrets_file_settings = Dynaconf(SETTINGS_FILES=[secrets_file])
    try:
        registry_token_expires_at = secrets_file_settings.EMBEDOPS_ECR_TOKEN_EXPIRES_AT
    except AttributeError as exc:
        raise UnauthorizedUserException from exc

    return registry_token_expires_at


def get_registry_token(secrets_file=user_secrets):
    """Retrieve the Auth0 token from user secrets"""
    if not os.path.exists(secrets_file):
        raise UnauthorizedUserException
    secrets_file_settings = Dynaconf(SETTINGS_FILES=[secrets_file])
    try:
        registry_token_id = secrets_file_settings.EMBEDOPS_ECR_TOKEN_ID
        registry_token_secret = secrets_file_settings.EMBEDOPS_ECR_TOKEN_SECRET
    except AttributeError as exc:
        raise UnauthorizedUserException from exc
    if not registry_token_id or not registry_token_secret:
        raise UnauthorizedUserException

    return {
        "registry_token_id": registry_token_id,
        "registry_token_secret": registry_token_secret,
    }


def set_registry_token(
    registry_access_key_id: str,
    registry_secret_access_key: str,
    secrets_file=user_secrets,
):
    """Set the AWS ECR registry access token in user secrets"""
    try:
        toml_loader.write(
            secrets_file, {"EMBEDOPS_ECR_TOKEN_ID": registry_access_key_id}, merge=True
        )
        toml_loader.write(
            secrets_file,
            {"EMBEDOPS_ECR_TOKEN_SECRET": registry_secret_access_key},
            merge=True,
        )

    except (IOError) as exc:
        raise LoginFailureException from exc


def auth0_url_encode(byte_data):
    """
    Safe encoding handles + and /, and also replace = with nothing
    :param byte_data:
    :return:
    """
    return base64.urlsafe_b64encode(byte_data).decode("utf-8").replace("=", "")


def request_authorization():  # pylint: disable=too-many-locals
    """start the routine for the user to authenticate with Auth0"""

    ############## Setup request bits ################
    env_path = pathlib.Path(".") / ".env"
    dotenv.load_dotenv(dotenv_path=env_path)

    # Uncomment to connect to platform in dev environment
    # todo: Need a make setting for switching between dev & prod env ticket: EO-473
    # client_id = "6Sm3VSVQ15B9IxTFKmwrjcPzXaH0oC1P"
    # base_url = "https://embedops-dev.us.auth0.com/oauth"

    client_id = "wBgMNZyLb76QRZe3eDRsCnoBLBy4qMQh"
    base_url = "https://embedops.us.auth0.com/oauth"

    code_obj = request_user_code(client_id, base_url)

    # Request the user code and verification url, show instructions to user
    user_code = code_obj["user_code"]
    show_login_instructions(user_code)

    # Open the browser window to the login url
    # Start the server and poll until the callback has been invoked
    verification_url = code_obj["verification_uri_complete"]
    device_code = code_obj["device_code"]

    return launch_login_url(verification_url, base_url, device_code, client_id)


def request_user_code(client_id, base_url):
    """Request the code to show to the user"""
    # We generate a nonce (state) that is used to protect against attackers invoking the callback

    # Uncomment to connect to platform in dev environment
    # todo: Need a make setting for switching between dev & prod env ticket: EO-473
    # audience = "https://app-dev.embedops.com"
    audience = "https://app.embedops.com"

    data = {
        "audience": audience,
        "scope": "openid email profile offline_access",
        "client_id": client_id,
    }

    url = f"{base_url}/device/code"

    code_req_response = requests.post(url, data=data)
    return code_req_response.json()


def show_login_instructions(user_code):
    """Printing some pretty instructions for the user while logging in"""
    click.secho("\n")
    click.secho("-" * 80, fg="magenta")
    click.secho("Welcome to Dojo Five EmbedOps CLI!\n", fg="magenta")
    click.secho(
        "To login, confirm this device in your browser by verifying this code:",
        fg="cyan",
    )
    click.secho(f"\n          >> {user_code} <<\n", fg="bright_cyan")
    click.secho("-" * 80, fg="magenta")


def launch_login_url(verification_url, base_url, device_code, client_id):
    """Launch the url for the user to log into EmbedOps"""
    webbrowser.open_new(verification_url)
    click.secho("\nWaiting for verification in browser:")
    click.secho("(To stop, press cancel in browser and type CTRL+C here)\n")
    verified = False
    while not verified:
        url = f"{base_url}/token"
        token_data = {
            "device_code": device_code,
            "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
            "client_id": client_id,
        }
        token_resp = None
        token_record = None
        try:
            token_resp = requests.post(url, data=token_data)
        except RuntimeError as exc:
            raise LoginFailureException from exc

        if token_resp:
            token_record = token_resp.json()
            click.secho("\n")
            break
        click.secho(".", nl=False, fg="magenta")
        sleep(0.1)

    return token_record["access_token"]


def check_token():
    """Check whether the token received is good or not"""
    user = None
    user_client = get_user_client()
    try:
        user = user_client.get_my_user()
    except (ValueError, TypeError, ApiException):
        return False

    return user is not None


def get_user_client():
    """Get a client for the embedops API as the currently signed in user"""
    api_host = settings.get("host")  # TODO: get config.py working in CLI
    auth_token = get_auth_token()
    configuration = Configuration()
    configuration.host = f"{api_host}/api/v1"
    configuration.api_key["Authorization"] = auth_token
    configuration.api_key_prefix["Authorization"] = "Bearer"

    api_client = ApiClient(configuration=configuration)

    return DefaultApi(api_client=api_client)


def docker_cli_login(aws_access_key_id, aws_secret_access_key):
    """Login using Docker CLI. Returns the return code of the subprocess"""

    sts_client = boto3.client(
        "sts",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
    )
    # need to assume proper role before being able to grab ECR credentials
    assume_role_response = sts_client.assume_role(
        RoleArn="arn:aws:iam::623731379476:role/EOToolsReadRole",
        RoleSessionName="EmbedOpsCLI-Session",
    )
    aws_access_key_id, aws_secret_access_key, aws_session_token = itemgetter(
        "AccessKeyId", "SecretAccessKey", "SessionToken"
    )(assume_role_response["Credentials"])
    ecr_client = boto3.client(
        "ecr",
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
        region_name="us-west-2",
    )

    ecr_authorization_data = ecr_client.get_authorization_token()["authorizationData"][
        0
    ]
    # We need to save this for later to check if the token is valid when we use 'docker run ...'
    set_docker_expiration_time(ecr_authorization_data["expiresAt"])

    authorization_token, registry_server = itemgetter(
        "authorizationToken", "proxyEndpoint"
    )(ecr_authorization_data)
    registry_token = base64.b64decode(authorization_token).decode().replace("AWS:", "")
    try:
        subprocess.check_output(
            f"echo {registry_token} | \
                docker login -u AWS --password-stdin {urlparse(registry_server).netloc}",
            shell=True,
            stderr=subprocess.DEVNULL,
        )
        return 0
    except subprocess.CalledProcessError as err:
        _logger.error(err.returncode)

        if "Is the docker daemon running?" in str(err.stdout):
            raise DockerNotRunningException from err

        return err.returncode


def login_to_registry(secrets_file=user_secrets):
    """Log into a docker registry"""
    try:
        registry_token_data = get_registry_token(secrets_file)

    except UnauthorizedUserException as exc:
        raise UnauthorizedUserException from exc
    return_code = docker_cli_login(
        registry_token_data["registry_token_id"],
        registry_token_data["registry_token_secret"],
    )
    if return_code == 1:
        raise DockerRegistryException
    if return_code > 0:
        raise UnknownDockerException
    return return_code


def is_registery_token_valid(secrets_file=user_secrets):
    """Check if register token has expired"""
    registry_token_expires_at = get_docker_expiration_time(secrets_file)
    is_valid = False

    if registry_token_expires_at:

        if registry_token_expires_at > datetime.now().timestamp():
            is_valid = True

    return is_valid



"""
`environment_utilities`
=======================================================================
Managing the environment variables for CLI context
* Author(s): Bailey Steinfadt
"""

import os
import re

ENV_FILE = ".env"
PROJECT_ENV_FILE = ".eo_env"
JOB_ENV_FILE = ".eo_job_env"


def _clean_env_file(env_file_contents: list):
    """clean env file for Docker"""
    clean_contents = []
    for line in env_file_contents:
        line = line.strip()
        if not line.startswith("#"):
            line = re.sub(r'=["\']', r"=", line)
            line = re.sub(r'["\']\s*$', r"", line)
            if len(line) > 0:
                clean_contents.append(f"{line}\n")
    return clean_contents


def _read_envvar_strings(filename):
    clean_env_contents = []

    if os.path.exists(filename):
        project_env_contents = ""
        with open(filename, "r", encoding="utf-8") as env_file:
            project_env_contents = env_file.readlines()

        clean_env_contents += _clean_env_file(project_env_contents)
    return clean_env_contents


# TODO: use a dynaconf loader.write to export the
#   config from dynaconf/internal settings to an env file that can be passed to docker
def create_job_env_file(yaml_vars: dict):
    """
    Collect environment variables from .eo_env and the run context
    and combine them into a file to be passed into the docker container.
    """
    delete_job_env_file()
    envvar_strings = []

    # envvar_strings += _read_envvar_strings(ENV_FILE)
    envvar_strings += _read_envvar_strings(PROJECT_ENV_FILE)
    envvar_strings += _read_envvar_strings(JOB_ENV_FILE)

    with open(JOB_ENV_FILE, "a", encoding="utf-8") as env_file:

        for line in envvar_strings:
            env_file.write(line)

        for key, value in yaml_vars.items():
            env_file.write(f"\n{key}={value}")


def delete_job_env_file():
    """Delete job env file"""
    if os.path.exists(JOB_ENV_FILE):
        os.remove(JOB_ENV_FILE)



"""
`docker_run`
=======================================================================
Module will take a local run context and run a job locally in Docker
* Author(s): Bailey Steinfadt
"""

import os
import re
import subprocess
import logging
from sys import platform

from embedops_cli.utilities import quote_str_for_platform
from embedops_cli import environment_utilities as envutil
from .eo_types import (
    NoDockerContainerException,
    InvalidDockerContainerException,
    DockerNotRunningException,
    DockerRegistryException,
    LocalRunContext,
)

MAX_CONTAINER_NAME_LENGTH = 30
_logger = logging.getLogger(__name__)


def _remove_special_char(string):
    """
    Remove special characters from the input string.
    """
    # Make an RE character set and pass it as an argument
    # into the compile function
    string_check = re.compile("[~`!@#$%^&*()+=}{\\[\\]|\\\\:;\"'<>?/,]")

    # Remove the special characters
    clean_string = string_check.sub("", string)

    return clean_string


def _clean_job_name(job_name):
    """
    Remove special characters, spaces from the input job name string,
    and truncate it if necessarily.
    """
    # Checkpoint 1: Check for disallowed characters and remove them.
    # Allowed characters: [a-zA-Z0-9][a-zA-Z0-9_.-]
    clean_job_name = _remove_special_char(job_name)

    # Remove spaces
    clean_job_name = clean_job_name.replace(" ", "")

    # Checkpoint 2: Check for the string length and truncate it if necessarily.
    # Container name can only be up to 30 characters long
    if len(clean_job_name) > MAX_CONTAINER_NAME_LENGTH:
        clean_job_name = clean_job_name[0:MAX_CONTAINER_NAME_LENGTH]

    return clean_job_name


# TODO: check that we have a script, docker_tag, and job_name
# TODO: add exceptions to eo_types and raise in here for different issues
def docker_run(run_context: LocalRunContext):
    """Takes a run context and launches Docker with the parameters"""

    if run_context.docker_tag == "":
        raise NoDockerContainerException()

    if "http://" in run_context.docker_tag:
        raise InvalidDockerContainerException()

    # We're assuming the tool is run from the same directory as the CI YAML
    _pwd = os.getcwd()
    container_name = _clean_job_name(run_context.job_name)

    _logger.debug(f"Current working directory: {_pwd}")
    _logger.debug(f"Clean container name: {container_name}")

    script = ";".join(run_context.script)

    _logger.debug(f"Script as string: {script}")

    envutil.create_job_env_file(run_context.variables)

    dockercmd = ["docker", "run", "--rm"]
    if os.path.exists(envutil.JOB_ENV_FILE):
        dockercmd.extend([f"--env-file={envutil.JOB_ENV_FILE}"])

    quoted_script = quote_str_for_platform(script)
    _docker_run_cmd_arg = f"/bin/bash -l -c {quoted_script}"

    dockercmd.extend(
        [
            "--name",
            container_name,
            "-v",
            f"{_pwd}:/eo_workdir",
            "-w",
            "/eo_workdir",
            "-e",
            f"EO_WORKDIR={_pwd}",
            "-v",
            "/var/run/docker.sock:/var/run/docker.sock",
        ]
        + (
            ["-e", "LOCAL_UID=$(id -u $USER)", "-e", "LOCAL_GID=$(id -g $USER)"]
            if platform in ("linux", "linux2")
            else []
        )
        + [
            "-i",
            run_context.docker_tag,
            _docker_run_cmd_arg,
        ]
    )

    _logger.debug(" ".join(dockercmd))

    _rc = None

    try:
        with subprocess.Popen(
            " ".join(dockercmd), shell=True, stdout=subprocess.PIPE
        ) as process:
            while True:
                output = process.stdout.readline()
                # used to check for empty output in Python2, but seems
                # to work with just poll in 2.7.12 and 3.5.2
                # if output == '' and process.poll() is not None:
                if process.poll() is not None:
                    break
                if output:
                    print(output.strip().decode())
        _rc = process.poll()
    except subprocess.CalledProcessError as err:

        _logger.error(f"Subprocess returned an error: {err}")
        # TODO: figure out how to use click to return these to the user
        # and logger to return things to the devs.
        try:
            print(err.stdout.decode("utf-8"))
        except UnicodeDecodeError as decode_err:
            _logger.error(decode_err)
        _logger.error(err.returncode)

        if "Is the docker daemon running?" in str(err.stdout):
            raise DockerNotRunningException from err
        if "Unable to find image" in str(err.stdout):
            raise DockerRegistryException from err

        return err.returncode
    finally:
        envutil.delete_job_env_file()
    return _rc



"""
`docker_run_test`
=======================================================================
Unit tests for the module that takes a local run context and run a job locally in Docker
* Author(s): Zhi Xuen Lai
"""
import pytest
from embedops_cli import docker_run
from embedops_cli.yaml_tools import bb_parser, gl_parser
from embedops_cli.eo_types import (
    NoDockerContainerException,
    InvalidDockerContainerException,
)
from tests import BBYML_FILENAME, GLYML_FILENAME

GLYML_NO_IMAGE_FILENAME = "tests/test-no-image-.gitlab-ci.yml"


def test_clean_bb_job_name_with_spaces():
    """Test cleaning Bitbucket job name with spaces"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "Step With Spaces":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithSpaces"


def test_clean_gl_job_name_with_spaces():
    """Test cleaning GitLab job name with spaces"""
    job_name_list = gl_parser.get_job_name_list(GLYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "Step With Spaces":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithSpaces"


def test_clean_bb_job_name_with_special_characters():
    """Test cleaning Bitbucket job name with special characters"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "~Step@With$Special>Char?":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithSpecialChar"


def test_clean_gl_job_name_with_special_characters():
    """Test cleaning GitLab job name with special characters"""
    job_name_list = gl_parser.get_job_name_list(GLYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "~Step@With$Special>Char?":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithSpecialChar"


def test_clean_bb_job_name_with_too_many_characters():
    """Test cleaning Bitbucket job name with too many characters"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "StepWithNameMoreThanThirtyCharacters":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithNameMoreThanThirtyChar"


def test_clean_gl_job_name_with_too_many_characters():
    """Test cleaning GitLab job name with too many characters"""
    job_name_list = gl_parser.get_job_name_list(GLYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "StepWithNameMoreThanThirtyCharacters":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithNameMoreThanThirtyChar"


def test_clean_bb_job_name_with_all_restrictions():
    """Test cleaning Bitbucket job name with all restrictions"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if (
            job_name
            == "Step% :With;&= All++([{Restrictions}])That* You> Can> |Imagine|!"
        ):
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithAllRestrictionsThatYou"


def test_clean_gl_job_name_with_all_restrictions():
    """Test cleaning GitLab job name with all restrictions"""
    job_name_list = gl_parser.get_job_name_list(GLYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if (
            job_name
            == "Step% :With;&= \\All++([{Restrictions}])That* You> Can> |Imagine|!"
        ):
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "StepWithAllRestrictionsThatYou"


def test_clean_bb_job_name_with_valid_name():
    """Test cleaning Bitbucket job name with valid name"""
    job_name_list = bb_parser.get_job_name_list(BBYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "St3p_W1th-Val1d.Nam3":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "St3p_W1th-Val1d.Nam3"


def test_clean_gl_job_name_with_valid_name():
    """Test cleaning GitLab job name with valid name"""
    job_name_list = gl_parser.get_job_name_list(GLYML_FILENAME)
    cleaned_job_name = ""

    for job_name in job_name_list:
        if job_name == "St3p_W1th-Val1d.Nam3":
            cleaned_job_name = docker_run._clean_job_name(job_name)
            break

    assert cleaned_job_name == "St3p_W1th-Val1d.Nam3"


def test_docker_run_without_image():
    """Test running a job with invalid image"""
    job_list = gl_parser.get_job_list(GLYML_NO_IMAGE_FILENAME)

    for job in job_list:
        if job.job_name == "StepWithoutImage":
            with pytest.raises(NoDockerContainerException):
                assert docker_run.docker_run(job)
            break


def test_docker_run_with_invalid_image():
    """Test running a job with invalid image"""
    job_list = gl_parser.get_job_list(GLYML_FILENAME)

    for job in job_list:
        if job.job_name == "StepWithInvalidImage":
            with pytest.raises(InvalidDockerContainerException):
                assert docker_run.docker_run(job)
            break



"""Regex pattern for parsing the output of `size` in the berkeley format"""
# regex pattern for arm-gcc-eabi-size in Berkeley format
# Example:
# https://regex101.com/r/f0wctA/1
# [ 99%] Linking C executable application.elf
#   text	   data	    bss	    dec	    hex	filename
#  130584	   2064	  69440	 202088	  31568	application.elf

# bss=>RAM
# data=>RAM (variables) + FLASH (initialization constants)
# text=> FLASH
SIZE_PATTERN = (
    r"text\s+data\s+bss\s+dec\s+hex\s+filename\n\s+"
    r"(?P<flash_code_size>\d+)\s+"
    r"(?P<flash_data_size>\d+)\s+"
    r"(?P<ram_size>\d+)\s+"
    r"(?P<dec>\d+)\s+"
    r"(?P<hex>[a-fA-F0-9]+)\s+"
    r"(?P<build_output_filename>\S+)"
)



"""
`gl_parser_test`
=======================================================================
Unit tests for the parser to pull job contexts from .gitlab-ci.yml files
* Author(s): Bailey Steinfadt
"""
import pytest
from embedops_cli.yaml_tools import gh_parser, yaml_utilities
from embedops_cli.eo_types import BadYamlFileException
from tests import GHYML_FILENAME


GHYML_INVALID_INDENTATION_FILENAME = "tests/test-invalid-indentation-.github-ci.yml"
GHYML_INVALID_JOB_NAME_FILENAME = "tests/test-invalid-job-name-.github-ci.yml"
GHYML_NO_STEPS_FILENAME = "tests/test-no-steps-.github-ci.yml"


def test_get_job_list_with_invalid_indentation_exception():
    """test that exception is raised for job with invalid indentation"""
    with pytest.raises(BadYamlFileException):
        assert gh_parser.get_job_list(GHYML_INVALID_INDENTATION_FILENAME)


def test_get_job_list_with_invalid_name_exception():
    """test that exception is raised for job with invalid name"""
    with pytest.raises(BadYamlFileException):
        assert gh_parser.get_job_list(GHYML_INVALID_JOB_NAME_FILENAME)


def test_get_job_list_without_steps_exception():
    """test that exception is raised for job without steps section"""
    with pytest.raises(BadYamlFileException):
        assert gh_parser.get_job_list(GHYML_NO_STEPS_FILENAME)


def test_get_job_list():
    """Test retrieving the list of complete local run context from the YAML"""
    job_list = gh_parser.get_job_list(GHYML_FILENAME)
    assert len(job_list) >= 3
    filtered_job_list = [
        job for job in job_list if job.job_name == "JobContainsOnlyLetters"
    ]
    assert len(filtered_job_list) == 1
    assert (
        filtered_job_list[0].docker_tag
        == "registry.embedops.com/dojofive/build-images/cppcheck"
    )
    assert filtered_job_list[0].script[0] == 'echo "Job contains only letters"'


def test_get_job_name_list_with_invalid_indentation_exception():
    """test that exception is raised for job with invalid indentation"""
    with pytest.raises(BadYamlFileException):
        assert gh_parser.get_job_name_list(GHYML_INVALID_INDENTATION_FILENAME)


def test_get_job_name_list_with_invalid_name_exception():
    """test that exception is raised for job with invalid name"""
    with pytest.raises(BadYamlFileException):
        assert gh_parser.get_job_name_list(GHYML_INVALID_JOB_NAME_FILENAME)


def test_get_job_name_list():
    """Test retrieving the list of job names from the YAML"""
    job_name_list = gh_parser.get_job_name_list(GHYML_FILENAME)
    assert len(job_name_list) >= 3
    assert (
        len(
            [
                job_name
                for job_name in job_name_list
                if job_name == "JobContainsOnlyLetters"
            ]
        )
        == 1
    )
    assert (
        len(
            [
                job_name
                for job_name in job_name_list
                if job_name == "_Job_Starts_With_Underscore"
            ]
        )
        == 1
    )
    assert (
        len(
            [
                job_name
                for job_name in job_name_list
                if job_name == "Job-Separated-By-Dash"
            ]
        )
        == 1
    )


def test_get_job_context():
    """Test getting the job context for an indicated job"""
    requested_name = "Job_Contains-A1123Valid-Characters"
    job = yaml_utilities.get_job_context_for_name(
        gh_parser, GHYML_FILENAME, requested_name
    )
    assert job.job_name == requested_name


def test_get_pipeline_variables():
    """Test getting variables defined outside of job definitions"""
    job_list = gh_parser.get_job_list(GHYML_FILENAME)
    job = [job for job in job_list if job.job_name == "Job1Contains2Numbers34567890"][0]
    # The job only has access to the PROJ_DIR variable
    assert "PROJ_DIR" in job.variables
    assert "MY_VAR" not in job.variables
    assert "MY_SUB_VAR" not in job.variables


def test_get_job_variables():
    """Test getting variables defined in a job"""
    job_list = gh_parser.get_job_list(GHYML_FILENAME)
    job = [job for job in job_list if job.job_name == "JobWithEnvVariable"][0]
    # The job only has access to the PROJ_DIR and MY_VAR variables
    assert "PROJ_DIR" in job.variables
    assert "MY_VAR" in job.variables
    assert "MY_SUB_VAR" not in job.variables


def test_get_step_variables():
    """Test getting variables defined in a step"""
    job_list = gh_parser.get_job_list(GHYML_FILENAME)
    job = [job for job in job_list if job.job_name == "JobWithStepLevelEnvVariable"][0]
    # The job only has access to the PROJ_DIR and MY_SUB_VAR variables
    assert "PROJ_DIR" in job.variables
    assert "MY_VAR" not in job.variables
    assert "MY_SUB_VAR" in job.variables


def test_job_with_run_in_sub_step():
    """Test getting run command defined in a sub-step"""
    job_list = gh_parser.get_job_list(GHYML_FILENAME)
    job = [job for job in job_list if job.job_name == "JobWithRunInSubstep"][0]
    assert job.script[0] == 'echo "Job with run in sub-step"'


def test_job_with_literal_multiline_block():
    """Test retrieving a literal multiline block"""
    job = yaml_utilities.get_job_context_for_name(
        gh_parser, GHYML_FILENAME, "JobWithLiteralMultilineBlock"
    )
    assert (
        job.script[0]
        == 'FILE=.clang-format\nif [ -f "$FILE" ]; then echo "$FILE exists. Use repository $FILE."; else echo "$FILE does not exist. Use container $FILE."; cp /tools/.clang-format .clang-format; fi\n'
    )


def test_step_with_folded_multiline_block():
    """Test retrieving a folded multiline block"""
    job = yaml_utilities.get_job_context_for_name(
        gh_parser, GHYML_FILENAME, "JobWithFoldedMultilineBlock"
    )
    assert (
        job.script[0]
        == 'FILE=.clang-format\nif [ -f "$FILE" ]; then echo "$FILE exists. Use repository $FILE."; else echo "$FILE does not exist. Use container $FILE."; cp /tools/.clang-format .clang-format; fi\n'
    )
